const Mocha = require('mocha');
const chai = require('chai');
const bit = require('bit-js');

const isEmptyObject = obj => Object.keys(obj).length === 0;

function normalizeResults(mochaJsonResults) {
  function normalizeError(err) {
    return {
      message: err.message,
      stack: err.stack
    };
  }

  function normalizeStats(stats) {
    return {
      start: stats.start,
      end: stats.end
    };
  }

  function normalizeTest(test) {
    const isError = !isEmptyObject(test.err);
    return ({
      title: test.fullTitle,
      pass: !isError,
      err: isError ? normalizeError(test.err) : null,
      duration: test.duration
    });
  }

  return {
    tests: mochaJsonResults.tests.map(normalizeTest),
    stats: normalizeStats(mochaJsonResults.stats)
  };
}

const run = (specFile) => {
  return new Promise((resolve) => {
    const mocha = new Mocha({ reporter: 'json' });
    mocha.addFile(specFile);
    mocha.run()
    .on('end', function() { // eslint-disable-line
      return resolve(normalizeResults(this.testResults));
    });
  });
};

const getTemplate = (name) => {
  return `const chai = require('chai');
const should = chai.should();
const component = require(__impl__);

describe('${name}', () => {
  it('the component should exist', () => {
    return should.exist(component);
  });
});

`;
};

module.exports = {
  run,
  globals: {
    chai,
    bit,
  },
  modules: {
    chai,
    'bit-js': bit,
  },
  getTemplate,
};
