'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = importAction;

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../bit-id');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../../consumer/component'));
}

var _consumer;

function _load_consumer() {
  return _consumer = _interopRequireDefault(require('../../../consumer/consumer'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function importAction(_ref) {
  let bitId = _ref.bitId,
      save = _ref.save,
      tester = _ref.tester,
      compiler = _ref.compiler,
      loader = _ref.loader,
      verbose = _ref.verbose,
      prefix = _ref.prefix;

  const performOnDir = prefix ? (_path || _load_path()).default.resolve(prefix) : process.cwd();
  return (_consumer || _load_consumer()).default.ensure(performOnDir).then(consumer => consumer.scope.ensureDir().then(() => consumer)).then(consumer => {
    if (tester || compiler) {
      loader.text = 'importing environment dependencies...';
      loader.start();

      return consumer.importEnvironment(bitId, verbose, loader).then(components => {
        function writeToBitJsonIfNeeded() {
          if (save && compiler) {
            consumer.bitJson.compilerId = components[0].id.toString();
            return consumer.bitJson.write({ bitDir: consumer.getPath() });
          }

          if (save && tester) {
            consumer.bitJson.testerId = components[0].id.toString();
            return consumer.bitJson.write({ bitDir: consumer.getPath() });
          }

          return Promise.resolve(true);
        }

        return writeToBitJsonIfNeeded().then(() => components);
      });
    }

    return consumer.import(bitId, verbose, loader).then(components => {
      if (save) {
        const parseId = (_bitId || _load_bitId()).BitId.parse(bitId, consumer.scope.name);
        return consumer.bitJson.addDependency(parseId).write({ bitDir: consumer.getPath() }).then(() => components);
      }

      return Promise.resolve(components);
    });
  });
}