'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.paintSpecsResults = exports.paintLog = exports.paintHeader = exports.paintBitProp = exports.formatBit = exports.formatInlineBit = undefined;

var _chalk;

function _load_chalk() {
  return _chalk = _interopRequireDefault(require('chalk'));
}

var _specsResults;

function _load_specsResults() {
  return _specsResults = _interopRequireDefault(require('../consumer/specs-results/specs-results'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const formatInlineBit = exports.formatInlineBit = (_ref) => {
  let box = _ref.box,
      name = _ref.name;
  return (_chalk || _load_chalk()).default.white('     > ') + (_chalk || _load_chalk()).default.cyan(`${box}/${name}`);
};

const formatBit = exports.formatBit = (_ref2) => {
  var _ref2$scope = _ref2.scope;
  let scope = _ref2$scope === undefined ? '@this' : _ref2$scope,
      box = _ref2.box,
      name = _ref2.name,
      version = _ref2.version;
  return (_chalk || _load_chalk()).default.white('     > ') + (_chalk || _load_chalk()).default.cyan(`${scope}/${box}/${name} - ${version ? version.toString() : 'latest'}`);
};

const paintBitProp = exports.paintBitProp = (key, value) => {
  if (!value) return '';
  return `${(_chalk || _load_chalk()).default.magenta(key)} -> ${value}\n`;
};

const paintHeader = exports.paintHeader = value => {
  if (!value) return '';
  return `${(_chalk || _load_chalk()).default.underline(value)}\n`;
};

const paintAuthor = (email, username) => {
  if (email && username) {
    return (_chalk || _load_chalk()).default.white(`Author: ${username} <${email}>\n`);
  } else if (email && !username) {
    return (_chalk || _load_chalk()).default.white(`Author: <${email}>\n`);
  } else if (!email && username) {
    return (_chalk || _load_chalk()).default.white(`Author: ${username}\n`);
  }

  return '';
};

const paintLog = exports.paintLog = (_ref3) => {
  let message = _ref3.message,
      date = _ref3.date,
      hash = _ref3.hash,
      username = _ref3.username,
      email = _ref3.email;

  return (_chalk || _load_chalk()).default.yellow(`commit ${hash}\n`) + paintAuthor(email, username) + (_chalk || _load_chalk()).default.white(`Date: ${date}\n`) + (_chalk || _load_chalk()).default.white(`\n      ${message}\n`);
};

const successTest = test => {
  return `✔   ${(_chalk || _load_chalk()).default.white(test.title)} - ${(_chalk || _load_chalk()).default.cyan(`${test.duration}ms`)}`;
};

const failureTest = test => {
  return `❌   ${(_chalk || _load_chalk()).default.white(test.title)} - ${(_chalk || _load_chalk()).default.cyan(`${test.duration}ms`)}
    ${(_chalk || _load_chalk()).default.red(test.err.message)}`;
};

const paintTest = test => {
  return test.pass ? successTest(test) : failureTest(test);
};

const paintStats = results => {
  const statsHeader = results.pass ? (_chalk || _load_chalk()).default.underline.green('tests passed') : (_chalk || _load_chalk()).default.underline.red('tests failed');
  const totalDuration = `total duration - ${(_chalk || _load_chalk()).default.cyan(`${results.stats.duration}ms\n`)}`;
  return `${statsHeader}\n${totalDuration}\n`;
};

const paintSpecsResults = exports.paintSpecsResults = results => {
  return paintStats(results) + results.tests.map(paintTest).join('\n');
};