'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class AbstractBitJson {

  constructor(_ref) {
    let impl = _ref.impl,
        spec = _ref.spec,
        compiler = _ref.compiler,
        tester = _ref.tester,
        dependencies = _ref.dependencies;

    this.impl = impl || (_constants || _load_constants()).DEFAULT_IMPL_NAME;
    this.spec = spec || (_constants || _load_constants()).DEFAULT_SPECS_NAME;
    this.compiler = compiler || (_constants || _load_constants()).DEFAULT_COMPILER_ID;
    this.tester = tester || (_constants || _load_constants()).DEFAULT_TESTER_ID;
    this.dependencies = dependencies || (_constants || _load_constants()).DEFAULT_DEPENDENCIES;
  }

  get compilerId() {
    return this.compiler;
  }

  set compilerId(compilerId) {
    this.compiler = compilerId;
  }

  get testerId() {
    return this.tester;
  }

  set testerId(testerId) {
    this.tester = testerId;
  }

  addDependency(bitId) {
    this.dependencies = (_ramda || _load_ramda()).default.merge(this.dependencies, bitId.toObject());
    return this;
  }

  getImplBasename() {
    return this.impl;
  }

  setImplBasename(name) {
    this.impl = name;
    return this;
  }

  getSpecBasename() {
    return this.spec;
  }

  setSpecBasename(name) {
    this.spec = name;
    return this;
  }

  hasCompiler() {
    return !!this.compiler && this.compiler !== (_constants || _load_constants()).NO_PLUGIN_TYPE;
  }

  hasTester() {
    return !!this.tester && this.tester !== (_constants || _load_constants()).NO_PLUGIN_TYPE;
  }

  getDependencies() {
    return (_bitId || _load_bitId()).BitIds.fromObject(this.dependencies);
  }

  toPlainObject() {
    return {
      sources: {
        impl: this.getImplBasename(),
        spec: this.getSpecBasename()
      },
      env: {
        compiler: this.compilerId,
        tester: this.testerId
      },
      dependencies: this.dependencies
    };
  }

  toJson() {
    let readable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

    if (!readable) return JSON.stringify(this.toPlainObject());
    return JSON.stringify(this.toPlainObject(), null, 4);
  }
}
exports.default = AbstractBitJson;