'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.composePath = composePath;
exports.hasExisting = hasExisting;

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fs;

function _load_fs() {
  return _fs = _interopRequireDefault(require('fs'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _abstractBitJson;

function _load_abstractBitJson() {
  return _abstractBitJson = _interopRequireDefault(require('./abstract-bit-json'));
}

var _consumerBitJson;

function _load_consumerBitJson() {
  return _consumerBitJson = _interopRequireDefault(require('./consumer-bit-json'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function composePath(bitPath) {
  return (_path || _load_path()).default.join(bitPath, (_constants || _load_constants()).BIT_JSON);
}
function hasExisting(bitPath) {
  return (_fs || _load_fs()).default.existsSync(composePath(bitPath));
}

class BitJson extends (_abstractBitJson || _load_abstractBitJson()).default {

  constructor(_ref) {
    let impl = _ref.impl,
        spec = _ref.spec,
        compiler = _ref.compiler,
        tester = _ref.tester,
        dependencies = _ref.dependencies,
        packageDependencies = _ref.packageDependencies;

    super({ impl, spec, compiler, tester, dependencies });
    this.packageDependencies = packageDependencies || {};
  }

  toPlainObject() {
    const superObject = super.toPlainObject();
    return (_ramda || _load_ramda()).default.merge(superObject, {
      packageDependencies: this.getPackageDependencies()
    });
  }

  getPackageDependencies() {
    return this.packageDependencies;
  }

  toJson() {
    let readable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

    if (!readable) return JSON.stringify(this.toPlainObject());
    return JSON.stringify(this.toPlainObject(), null, 4);
  }

  write(_ref2) {
    let bitDir = _ref2.bitDir;
    var _ref2$override = _ref2.override;
    let override = _ref2$override === undefined ? true : _ref2$override;

    return new Promise((resolve, reject) => {
      if (!override && hasExisting(bitDir)) {
        throw new (_exceptions || _load_exceptions()).BitJsonAlreadyExists();
      }

      const repspond = (err, res) => {
        if (err) return reject(err);
        return resolve(res);
      };

      (_fs || _load_fs()).default.writeFile(composePath(bitDir), this.toJson(), repspond);
    });
  }

  validate() {
    if (typeof this.getImplBasename() !== 'string' || typeof this.compilerId !== 'string' || typeof this.testerId !== 'string' || typeof this.getDependencies() !== 'object') throw new (_exceptions || _load_exceptions()).InvalidBitJson();
  }

  static fromPlainObject(object) {
    const sources = object.sources,
          env = object.env,
          dependencies = object.dependencies,
          packageDependencies = object.packageDependencies;

    return new BitJson({
      impl: (_ramda || _load_ramda()).default.prop('impl', sources),
      spec: (_ramda || _load_ramda()).default.prop('spec', sources),
      compiler: (_ramda || _load_ramda()).default.prop('compiler', env),
      tester: (_ramda || _load_ramda()).default.prop('tester', env),
      dependencies,
      packageDependencies
    });
  }

  static mergeWithProto(json, protoBJ) {
    const plainProtoBJ = protoBJ ? protoBJ.toPlainObject() : {};
    delete plainProtoBJ.dependencies;

    return BitJson.fromPlainObject((_ramda || _load_ramda()).default.merge(plainProtoBJ, json));
  }

  static create() {
    let json = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let protoBJ = arguments[1];

    return this.mergeWithProto(json, protoBJ);
  }

  static load(dirPath, protoBJ) {
    return new Promise(resolve => {
      let thisBJ = {};
      try {
        thisBJ = JSON.parse((_fs || _load_fs()).default.readFileSync(composePath(dirPath)).toString('utf8'));
      } catch (e) {} // eslint-disable-line

      const mergedBJ = this.mergeWithProto(thisBJ, protoBJ);
      return resolve(mergedBJ);
    });
  }

  static loadSync(dirPath, protoBJ) {
    let thisBJ = {};
    try {
      thisBJ = JSON.parse((_fs || _load_fs()).default.readFileSync(composePath(dirPath)).toString('utf8'));
    } catch (e) {} // eslint-disable-line

    return this.mergeWithProto(thisBJ, protoBJ);
  }
}
exports.default = BitJson;