'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _bitJson;

function _load_bitJson() {
  return _bitJson = _interopRequireDefault(require('../bit-json'));
}

var _impl;

function _load_impl() {
  return _impl = _interopRequireDefault(require('../component/sources/impl'));
}

var _specs;

function _load_specs() {
  return _specs = _interopRequireDefault(require('../component/sources/specs'));
}

var _dist;

function _load_dist() {
  return _dist = _interopRequireDefault(require('../component/sources/dist'));
}

var _consumerBitJson;

function _load_consumerBitJson() {
  return _consumerBitJson = _interopRequireDefault(require('../bit-json/consumer-bit-json'));
}

var _bitId;

function _load_bitId() {
  return _bitId = _interopRequireDefault(require('../../bit-id/bit-id'));
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../../scope/scope'));
}

var _bitIds;

function _load_bitIds() {
  return _bitIds = _interopRequireDefault(require('../../bit-id/bit-ids'));
}

var _environment;

function _load_environment() {
  return _environment = _interopRequireDefault(require('../../scope/repositories/environment'));
}

var _parser;

function _load_parser() {
  return _parser = _interopRequireDefault(require('../../jsdoc/parser'));
}

var _parser2;

function _load_parser2() {
  return _parser2 = require('../../jsdoc/parser');
}

var _specsRunner;

function _load_specsRunner() {
  return _specsRunner = _interopRequireDefault(require('../../specs-runner'));
}

var _specsResults;

function _load_specsResults() {
  return _specsResults = _interopRequireDefault(require('../specs-results'));
}

var _componentSpecsFailed;

function _load_componentSpecsFailed() {
  return _componentSpecsFailed = _interopRequireDefault(require('../exceptions/component-specs-failed'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Component {

  set impl(val) {
    this._impl = val;
  }

  get impl() {
    if ((0, (_utils || _load_utils()).isString)(this._impl)) {
      // $FlowFixMe
      this._impl = (_impl || _load_impl()).default.load(this._impl);
    }
    // $FlowFixMe
    return this._impl;
  }

  set specs(val) {
    this._specs = val;
  }

  get specs() {
    if (!this._specs) return null;

    if ((0, (_utils || _load_utils()).isString)(this._specs)) {
      // $FlowFixMe
      this._specs = (_specs || _load_specs()).default.load(this._specs);
    }
    // $FlowFixMe
    return this._specs;
  }

  get id() {
    if (!this.scope || !this.version) {
      console.error(this);
      throw new Error('cant produce id because scope or version are missing');
    }

    return new (_bitId || _load_bitId()).default({
      scope: this.scope,
      box: this.box,
      name: this.name,
      version: this.version.toString()
    });
  }

  get docs() {
    if (!this._docs) this._docs = (0, (_parser || _load_parser()).default)(this.impl.src);
    return this._docs;
  }

  constructor(_ref) {
    let name = _ref.name,
        box = _ref.box,
        version = _ref.version,
        scope = _ref.scope,
        implFile = _ref.implFile,
        specsFile = _ref.specsFile,
        compilerId = _ref.compilerId,
        testerId = _ref.testerId,
        dependencies = _ref.dependencies,
        packageDependencies = _ref.packageDependencies,
        impl = _ref.impl,
        specs = _ref.specs,
        docs = _ref.docs,
        dist = _ref.dist,
        specsResults = _ref.specsResults;

    this.name = name;
    this.box = box || (_constants || _load_constants()).DEFAULT_BOX_NAME;
    this.version = version;
    this.scope = scope;
    this.implFile = implFile || (_constants || _load_constants()).DEFAULT_IMPL_NAME;
    this.specsFile = specsFile || (_constants || _load_constants()).DEFAULT_SPECS_NAME;
    this.compilerId = compilerId;
    this.testerId = testerId;
    this.dependencies = dependencies || new (_bitIds || _load_bitIds()).default();
    this.packageDependencies = packageDependencies || {};
    this._specs = specs;
    this._impl = impl;
    this._docs = docs;
    this.dist = dist;
    this.specsResults = specsResults;
  }

  writeBitJson(bitDir) {
    return new (_bitJson || _load_bitJson()).default({
      version: this.version,
      scope: this.scope,
      impl: this.implFile,
      spec: this.specsFile,
      compiler: this.compilerId ? this.compilerId.toString() : (_constants || _load_constants()).NO_PLUGIN_TYPE,
      tester: this.testerId ? this.testerId.toString() : (_constants || _load_constants()).NO_PLUGIN_TYPE,
      dependencies: this.dependencies.toObject(),
      packageDependencies: this.packageDependencies
    }).write({ bitDir });
  }

  dependencies() {
    return (_bitIds || _load_bitIds()).default.fromObject(this.dependencies);
  }

  write(bitDir, withBitJson) {
    return (0, (_utils || _load_utils()).mkdirp)(bitDir).then(() => this.impl.write(bitDir, this.implFile)).then(() => {
      return this.specs ? this.specs.write(bitDir, this.specsFile) : undefined;
    }).then(() => {
      return this.dist ? this.dist.write(bitDir) : undefined;
    }).then(() => {
      return withBitJson ? this.writeBitJson(bitDir) : undefined;
    }).then(() => this);
  }

  runSpecs(scope, rejectOnFailure) {
    function compileIfNeeded(condition, compiler, src) {
      if (!condition || !compiler || !compiler.compile) return src;
      return compiler.compile(src).code;
    }

    if (!this.testerId || !this.specs || !this.specs.src) return Promise.resolve(null);

    try {
      const testerFilePath = scope.loadEnvironment(this.testerId, { pathOnly: true });
      const compiler = this.compilerId ? scope.loadEnvironment(this.compilerId) : null;
      const implSrc = compileIfNeeded(!!this.compilerId, compiler, this.impl.src);
      // $FlowFixMe
      const specsSrc = compileIfNeeded(!!this.compilerId, compiler, this.specs.src);
      return (_specsRunner || _load_specsRunner()).default.run({ scope, testerFilePath, implSrc, specsSrc }).then(specsResults => {
        this.specsResults = (_specsResults || _load_specsResults()).default.createFromRaw(specsResults);
        if (rejectOnFailure && !this.specsResults.pass) {
          return Promise.reject(new (_componentSpecsFailed || _load_componentSpecsFailed()).default());
        }

        return this.specsResults;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }

  build(scope) {
    // @TODO - write SourceMap Type
    if (!this.compilerId) return null;
    try {
      const compiler = scope.loadEnvironment(this.compilerId);
      const src = this.impl.src;

      var _compiler$compile = compiler.compile(src);

      const code = _compiler$compile.code,
            map = _compiler$compile.map; // eslint-disable-line

      this.dist = new (_dist || _load_dist()).default(code);
      return code;
    } catch (e) {
      throw e;
      // return null;
    }
  }

  toObject() {
    return {
      name: this.name,
      box: this.box,
      version: this.version ? this.version.toString() : null,
      scope: this.scope,
      implFile: this.implFile,
      specsFile: this.specsFile,
      compilerId: this.compilerId ? this.compilerId.toString() : null,
      testerId: this.testerId ? this.testerId.toString() : null,
      dependencies: this.dependencies.toObject(),
      packageDependencies: this.packageDependencies,
      specs: this.specs ? this.specs.serialize() : null,
      impl: this.impl.serialize(),
      docs: this.docs,
      dist: this.dist ? this.dist.serialize() : null,
      specsResults: this.specsResults ? this.specsResults.serialize() : null
    };
  }

  toString() {
    return JSON.stringify(this.toObject());
  }

  static fromObject(object) {
    const name = object.name,
          box = object.box,
          version = object.version,
          scope = object.scope,
          implFile = object.implFile,
          specsFile = object.specsFile,
          compilerId = object.compilerId,
          testerId = object.testerId,
          dependencies = object.dependencies,
          packageDependencies = object.packageDependencies,
          impl = object.impl,
          specs = object.specs,
          docs = object.docs,
          dist = object.dist,
          specsResults = object.specsResults;


    return new Component({
      name,
      box,
      version: parseInt(version),
      scope,
      implFile,
      specsFile,
      compilerId: compilerId ? (_bitId || _load_bitId()).default.parse(compilerId) : null,
      testerId: testerId ? (_bitId || _load_bitId()).default.parse(testerId) : null,
      dependencies: (_bitIds || _load_bitIds()).default.fromObject(dependencies),
      packageDependencies,
      impl: (_impl || _load_impl()).default.deserialize(impl),
      specs: specs ? (_specs || _load_specs()).default.deserialize(specs) : null,
      docs,
      dist: dist ? (_dist || _load_dist()).default.deserialize(dist) : null,
      specsResults: specsResults ? (_specsResults || _load_specsResults()).default.deserialize(specsResults) : null
    });
  }

  static fromString(str) {
    const object = JSON.parse(str);
    return this.fromObject(object);
  }

  static loadFromInline(bitDir, consumerBitJson) {
    return (_bitJson || _load_bitJson()).default.load(bitDir, consumerBitJson).then(bitJson => {
      return new Component({
        name: (_path || _load_path()).default.basename(bitDir),
        box: (_path || _load_path()).default.basename((_path || _load_path()).default.dirname(bitDir)),
        implFile: bitJson.getImplBasename(),
        specsFile: bitJson.getSpecBasename(),
        compilerId: (_bitId || _load_bitId()).default.parse(bitJson.compilerId),
        testerId: (_bitId || _load_bitId()).default.parse(bitJson.testerId),
        dependencies: (_bitIds || _load_bitIds()).default.fromObject(bitJson.dependencies),
        packageDependencies: bitJson.packageDependencies,
        impl: (_path || _load_path()).default.join(bitDir, bitJson.getImplBasename()),
        specs: (_path || _load_path()).default.join(bitDir, bitJson.getSpecBasename())
      });
    });
  }

  static create(_ref2, scope) {
    let scopeName = _ref2.scopeName,
        name = _ref2.name,
        box = _ref2.box,
        withSpecs = _ref2.withSpecs,
        consumerBitJson = _ref2.consumerBitJson;

    const implFile = consumerBitJson.getImplBasename();
    const specsFile = consumerBitJson.getSpecBasename();
    const compilerId = (_bitId || _load_bitId()).default.parse(consumerBitJson.compilerId);
    const testerId = (_bitId || _load_bitId()).default.parse(consumerBitJson.testerId);

    return new Component({
      name,
      box,
      version: (_constants || _load_constants()).DEFAULT_BIT_VERSION,
      scope: scopeName,
      implFile,
      specsFile,
      compilerId,
      testerId,
      impl: (_impl || _load_impl()).default.create(name, compilerId, scope),
      specs: withSpecs ? (_specs || _load_specs()).default.create(name, testerId, scope) : undefined
    });
  }
}
exports.default = Component;