'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs;

function _load_fs() {
  return _fs = _interopRequireWildcard(require('fs'));
}

var _path;

function _load_path() {
  return _path = _interopRequireWildcard(require('path'));
}

var _source;

function _load_source() {
  return _source = _interopRequireDefault(require('./source'));
}

var _impl;

function _load_impl() {
  return _impl = _interopRequireDefault(require('../templates/impl.default-template'));
}

var _bitId;

function _load_bitId() {
  return _bitId = _interopRequireDefault(require('../../../bit-id'));
}

var _invalidImpl;

function _load_invalidImpl() {
  return _invalidImpl = _interopRequireDefault(require('../exceptions/invalid-impl'));
}

var _missingImpl;

function _load_missingImpl() {
  return _missingImpl = _interopRequireDefault(require('../exceptions/missing-impl'));
}

var _scope;

function _load_scope() {
  return _scope = require('../../../scope');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

class Impl extends (_source || _load_source()).default {
  write(bitPath, fileName) {
    return new Promise((resolve, reject) => (_fs || _load_fs()).writeFile((_path || _load_path()).join(bitPath, fileName), this.src, (err, res) => {
      if (err) return reject(err);
      return resolve(res);
    }));
  }

  validate() {
    if (typeof this.src !== 'string') throw new (_invalidImpl || _load_invalidImpl()).default();
  }

  serialize() {
    return this.src;
  }

  static deserialize(str) {
    return new Impl(str);
  }

  static load(implPath) {
    try {
      const data = (_fs || _load_fs()).readFileSync(implPath);
      return new Impl(data.toString());
    } catch (err) {
      if (err.code === 'ENOENT') {
        throw new (_missingImpl || _load_missingImpl()).default(implPath);
      }
      throw err;
    }
  }

  static create(name, compilerId, scope) {
    function getTemplate() {
      try {
        const testerModule = scope.loadEnvironment(compilerId);
        return testerModule.getTemplate(name);
      } catch (e) {
        return (0, (_impl || _load_impl()).default)({ name });
      }
    }

    return new Impl(getTemplate());
  }
}
exports.default = Impl;