'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _glob;

function _load_glob() {
  return _glob = _interopRequireDefault(require('glob'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _flattenDependencies;

function _load_flattenDependencies() {
  return _flattenDependencies = require('../scope/flatten-dependencies');
}

var _consumerLocator;

function _load_consumerLocator() {
  return _consumerLocator = require('./consumer-locator');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _consumerBitJson;

function _load_consumerBitJson() {
  return _consumerBitJson = _interopRequireDefault(require('./bit-json/consumer-bit-json'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('./component'));
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _scope;

function _load_scope() {
  return _scope = require('../scope');
}

var _bitInlineId;

function _load_bitInlineId() {
  return _bitInlineId = _interopRequireDefault(require('./bit-inline-id'));
}

var _indexer;

function _load_indexer() {
  return _indexer = require('../search/indexer');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Consumer {

  constructor(_ref) {
    let projectPath = _ref.projectPath,
        bitJson = _ref.bitJson,
        scope = _ref.scope;
    var _ref$created = _ref.created;
    let created = _ref$created === undefined ? false : _ref$created;

    this.projectPath = projectPath;
    this.bitJson = bitJson;
    this.created = created;
    this.scope = scope;
  }

  get testerId() {
    return (_bitId || _load_bitId()).BitId.parse(this.bitJson.testerId, this.scope);
  }

  get compilerId() {
    return (_bitId || _load_bitId()).BitId.parse(this.bitJson.compilerId, this.scope);
  }

  write() {
    return this.bitJson.write({ bitDir: this.projectPath }).then(() => this.scope.ensureDir()).then(() => this);
  }

  getInlineBitsPath() {
    return (_path || _load_path()).default.join(this.projectPath, (_constants || _load_constants()).INLINE_BITS_DIRNAME);
  }

  getComponentsPath() {
    return (_path || _load_path()).default.join(this.projectPath, (_constants || _load_constants()).BITS_DIRNAME);
  }

  getPath() {
    return this.projectPath;
  }

  loadComponent(id) {
    const bitDir = id.composeBitPath(this.getPath());
    return (_component || _load_component()).default.loadFromInline(bitDir, this.bitJson);
  }

  exportAction(rawId, rawRemote) {
    // @TODO - move this method to api, not related to consumer
    const bitId = (_bitId || _load_bitId()).BitId.parse(rawId, this.scope.name);

    return this.scope.exportAction(bitId, rawRemote).then(componentDependencies => this.writeToComponentsDir([componentDependencies]).then(() => this.removeFromComponents(bitId.changeScope(this.scope.name))) // @HACKALERT
    .then(() => componentDependencies.component));
  }

  import(rawId, verbose, loader, withEnvironments) {
    const importAccordingToConsumerBitJson = () => {
      const dependencies = (_bitId || _load_bitId()).BitIds.fromObject(this.bitJson.dependencies);
      if ((_ramda || _load_ramda()).default.isNil(dependencies) || (_ramda || _load_ramda()).default.isEmpty(dependencies)) {
        return Promise.reject(new (_exceptions || _load_exceptions()).NothingToImport());
      }

      if (loader) loader.start();
      return this.scope.getMany(dependencies).then(components => {
        return this.writeToComponentsDir((0, (_utils || _load_utils()).flatten)(components)).then(depComponents => {
          return withEnvironments ? this.scope.installEnvironment({
            ids: [this.testerId, this.compilerId],
            consumer: this,
            verbose,
            loader
          }).then(envComponents => (_ramda || _load_ramda()).default.concat(depComponents, envComponents)) : depComponents;
        });
      });
    };

    const importSpecificComponent = () => {
      const bitId = (_bitId || _load_bitId()).BitId.parse(rawId, this.scope.name);
      if (loader) loader.start();
      return this.scope.get(bitId).then(component => {
        return this.writeToComponentsDir([component]);
      });
    };

    if (!rawId) return importAccordingToConsumerBitJson();
    return importSpecificComponent();
  }

  importEnvironment(rawId, verbose, loader) {
    if (!rawId) {
      throw new Error('you must specify bit id for importing');
    } // @TODO - make a normal error message
    const bitId = (_bitId || _load_bitId()).BitId.parse(rawId, this.scope.name);
    return this.scope.installEnvironment({ ids: [bitId], consumer: this, verbose, loader });
  }

  createBit(_ref2) {
    let id = _ref2.id;
    var _ref2$withSpecs = _ref2.withSpecs;
    let withSpecs = _ref2$withSpecs === undefined ? false : _ref2$withSpecs;
    var _ref2$withBitJson = _ref2.withBitJson;
    let withBitJson = _ref2$withBitJson === undefined ? false : _ref2$withBitJson;

    const inlineBitPath = id.composeBitPath(this.getPath());

    return (_component || _load_component()).default.create({
      name: id.name,
      box: id.box,
      withSpecs,
      consumerBitJson: this.bitJson
    }, this.scope).write(inlineBitPath, withBitJson);
  }

  removeFromInline(id) {
    const componentDir = id.composeBitPath(this.getPath());
    return new Promise((resolve, reject) => {
      return (_fsExtra || _load_fsExtra()).default.remove(componentDir, err => {
        if (err) return reject(err);
        return (0, (_utils || _load_utils()).removeContainingDirIfEmpty)(componentDir).then(resolve);
      });
    });
  }

  removeFromComponents(id) {
    // @TODO - also consider
    // @HACKALERT - also consider version when removing a directory from components

    const componentsDir = this.getComponentsPath();
    const componentDir = (_path || _load_path()).default.join(componentsDir, id.box, id.name, id.scope);

    return new Promise((resolve, reject) => {
      return (_fsExtra || _load_fsExtra()).default.remove(componentDir, err => {
        if (err) return reject(err);
        return resolve();
      });
    });
  }

  writeToComponentsDir(componentDependencies) {
    const componentsDir = this.getComponentsPath();
    const components = (0, (_flattenDependencies || _load_flattenDependencies()).flattenDependencies)(componentDependencies);

    const bitDirForConsumerImport = component => {
      return (_path || _load_path()).default.join(componentsDir, component.box, component.name, component.scope, component.version.toString());
    };

    return Promise.all(components.map(component => {
      const bitPath = bitDirForConsumerImport(component);
      return component.write(bitPath, true);
    }));
  }

  commit(id, message, force, loader) {
    return this.loadComponent(id).then(bit => this.scope.put(bit, message, force, loader).then(bits => this.writeToComponentsDir([bits])).then(() => this.removeFromInline(id)).then(() => (0, (_indexer || _load_indexer()).index)(bit, this.scope.getPath())).then(() => bit));
  }

  runComponentSpecs(id) {
    return this.loadComponent(id).then(component => {
      return component.runSpecs(this.scope);
    });
  }

  listInline() {
    return new Promise((resolve, reject) => (0, (_glob || _load_glob()).default)((_path || _load_path()).default.join('*', '*'), { cwd: this.getInlineBitsPath() }, (err, files) => {
      if (err) reject(err);

      const bitsP = files.map(bitRawId => this.loadComponent((_bitInlineId || _load_bitInlineId()).default.parse(bitRawId)));

      return Promise.all(bitsP).then(resolve).catch(reject);
    }));
  }

  includes(_ref3) {
    let inline = _ref3.inline,
        bitName = _ref3.bitName;

    const dirToCheck = inline ? this.getInlineBitsPath() : this.getComponentsPath();

    return new Promise(resolve => {
      return (_fsExtra || _load_fsExtra()).default.stat((_path || _load_path()).default.join(dirToCheck, bitName), err => {
        if (err) return resolve(false);
        return resolve(true);
      });
    });
  }

  static create() {
    let projectPath = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();

    if ((0, (_consumerLocator || _load_consumerLocator()).pathHasConsumer)(projectPath)) throw new (_exceptions || _load_exceptions()).ConsumerAlreadyExists();
    return this.ensure(projectPath);
  }

  static ensure() {
    let projectPath = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();

    const scopeP = (_scope || _load_scope()).Scope.ensure((_path || _load_path()).default.join(projectPath, (_constants || _load_constants()).BIT_HIDDEN_DIR));
    const bitJsonP = (_consumerBitJson || _load_consumerBitJson()).default.ensure(projectPath);

    return Promise.all([scopeP, bitJsonP]).then((_ref4) => {
      var _ref5 = _slicedToArray(_ref4, 2);

      let scope = _ref5[0],
          bitJson = _ref5[1];

      return new Consumer({
        projectPath,
        created: true,
        scope,
        bitJson
      });
    });
  }

  static load(currentPath) {
    return new Promise((resolve, reject) => {
      const projectPath = (0, (_consumerLocator || _load_consumerLocator()).locateConsumer)(currentPath);
      if (!projectPath) return reject(new (_exceptions || _load_exceptions()).ConsumerNotFound());
      const scopeP = (_scope || _load_scope()).Scope.load((_path || _load_path()).default.join(projectPath, (_constants || _load_constants()).BIT_HIDDEN_DIR));
      const bitJsonP = (_consumerBitJson || _load_consumerBitJson()).default.load(projectPath);
      return Promise.all([scopeP, bitJsonP]).then((_ref6) => {
        var _ref7 = _slicedToArray(_ref6, 2);

        let scope = _ref7[0],
            bitJson = _ref7[1];
        return resolve(new Consumer({
          projectPath,
          bitJson,
          scope
        }));
      });
    });
  }
}
exports.default = Consumer;