'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fs;

function _load_fs() {
  return _fs = _interopRequireDefault(require('fs'));
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPath() {
  return (_path || _load_path()).default.join((_constants || _load_constants()).GLOBAL_CONFIG, (_constants || _load_constants()).GLOBAL_CONFIG_FILE);
}

class Config extends Map {
  toPlainObject() {
    return (0, (_utils || _load_utils()).mapToObject)(this);
  }

  toJson() {
    return JSON.stringify(this.toPlainObject());
  }

  write() {
    return (0, (_utils || _load_utils()).writeFile)(getPath(), this.toJson());
  }

  writeSync() {
    return (_fs || _load_fs()).default.writeFileSync(getPath(), this.toJson());
  }

  static loadSync() {
    try {
      const contents = (_fs || _load_fs()).default.readFileSync(getPath());
      return new Config((0, (_utils || _load_utils()).objectToTupleArray)(JSON.parse(contents.toString())));
    } catch (err) {
      if (err.code !== 'ENOENT') return err;
      const config = new Config([]);
      config.writeSync();
      return config;
    }
  }

  static load() {
    return (0, (_utils || _load_utils()).readFile)(getPath()).then(contents => new Config((0, (_utils || _load_utils()).objectToTupleArray)(JSON.parse(contents.toString())))).catch(err => {
      if (err.code !== 'ENOENT') return err;
      const config = new Config([]);
      return config.write().then(() => config);
    });
  }
}
exports.default = Config;