'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('../scope/component-objects'));
}

var _network;

function _load_network() {
  return _network = require('../scope/network');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @ctx bit, primary, remote
 */
function isPrimary(alias) {
  return (0, (_utils || _load_utils()).contains)(alias, '!');
}
class Remote {

  constructor(host, name) {
    let primary = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
    this.primary = false;

    this.name = name || '';
    this.host = host;
    this.primary = primary;
  }

  connect() {
    return (0, (_network || _load_network()).connect)(this.host);
  }

  toPlainObject() {
    return {
      host: this.host,
      name: this.name
    };
  }

  scope() {
    return this.connect().then(network => {
      return network.describeScope();
    });
  }

  list() {
    return this.connect().then(network => network.list());
  }

  search(query, reindex) {
    return this.connect().then(network => network.search(query, reindex));
  }

  show(bitId) {
    return this.connect().then(network => network.show(bitId));
  }

  fetch(bitIds) {
    return this.connect().then(network => network.fetch(bitIds));
  }

  fetchOnes(ids) {
    return this.connect().then(network => network.fetch(ids, true));
  }

  validate() {
    if (!(0, (_utils || _load_utils()).isBitUrl)(this.host)) throw new (_exceptions || _load_exceptions()).InvalidRemote();
  }

  push(componentObjects) {
    return (0, (_network || _load_network()).connect)(this.host).then(network => network.push(componentObjects));
  }

  static load(name, host) {
    const primary = isPrimary(name);
    if (primary) name = (0, (_utils || _load_utils()).cleanBang)(name);

    return new Remote(name, host, primary);
  }
}
exports.default = Remote;