'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.DependencyMap = undefined;
exports.getPath = getPath;

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('../repository'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope'));
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../consumer/component'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPath(repositoryPath) {
  return (_path || _load_path()).default.join(repositoryPath, (_constants || _load_constants()).DEPENDENCY_MAP_FILENAME);
}
class DependencyMap extends Map {

  constructor(repository) {
    let dependencyTuples = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    super(dependencyTuples);
    this.repository = repository;
  }

  get(bitId) {
    return super.get(bitId.toString());
  }

  getBitIds(dependencies) {
    return new (_bitId || _load_bitId()).BitIds(...dependencies);
  }

  toObject() {
    const obj = {};
    this.forEach((bitIds, bitId) => {
      obj[bitId.toString()] = bitIds.map(dependency => dependency.toString());
    });
    return obj;
  }

  getPath() {
    return (_path || _load_path()).default.join(this.repository.getPath(), (_constants || _load_constants()).DEPENDENCY_MAP_FILENAME);
  }

  write() {
    return (0, (_utils || _load_utils()).writeFile)(this.getPath(), JSON.stringify(this.toObject()));
  }

  setBit(id, bits) {
    super.set(id.toString(), new (_bitId || _load_bitId()).BitIds(...bits.map(bit => bit.getId())));
    return this;
  }

  static load(json, repository) {
    const matrix = [];
    (0, (_utils || _load_utils()).forEach)(json, (val, key) => {
      matrix.push([key, val.map(bitDep => (_bitId || _load_bitId()).BitId.parse(bitDep.id))]);
    });

    return new DependencyMap(repository, matrix);
  }
}
exports.DependencyMap = DependencyMap;