'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objects;

function _load_objects() {
  return _objects = require('../objects');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope'));
}

var _source;

function _load_source() {
  return _source = _interopRequireDefault(require('./source'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../consumer/component'));
}

var _remotes;

function _load_remotes() {
  return _remotes = require('../../remotes');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _componentVersion;

function _load_componentVersion() {
  return _componentVersion = _interopRequireDefault(require('../component-version'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Version extends (_objects || _load_objects()).BitObject {

  constructor(_ref) {
    let impl = _ref.impl,
        specs = _ref.specs,
        dist = _ref.dist,
        compiler = _ref.compiler,
        tester = _ref.tester,
        log = _ref.log,
        dependencies = _ref.dependencies,
        docs = _ref.docs,
        specsResults = _ref.specsResults,
        flattenedDependencies = _ref.flattenedDependencies,
        packageDependencies = _ref.packageDependencies;

    super();
    this.impl = impl;
    this.specs = specs;
    this.dist = dist;
    this.compiler = compiler;
    this.tester = tester;
    this.log = log;
    this.dependencies = dependencies || new (_bitId || _load_bitId()).BitIds();
    this.docs = docs;
    this.specsResults = specsResults;
    this.flattenedDependencies = flattenedDependencies || new (_bitId || _load_bitId()).BitIds();
    this.packageDependencies = packageDependencies || {};
  }

  id() {
    return JSON.stringify(this.toObject());
  }

  collectDependencies(scope) {
    return scope.importManyOnes(this.flattenedDependencies);
  }

  refs() {
    return [this.impl.file,
    // $FlowFixMe
    this.specs ? this.specs.file : null,
    // $FlowFixMe (after filtering the nulls there is no problem)
    this.dist ? this.dist.file : null].filter(ref => ref);
  }

  toObject() {
    return (0, (_utils || _load_utils()).filterObject)({
      impl: {
        file: this.impl.file.toString(),
        name: this.impl.name
      },
      specs: this.specs ? {
        file: this.specs.file.toString(),
        // $FlowFixMe
        name: this.specs.name
      } : null,
      dist: this.dist ? {
        file: this.dist.file.toString(),
        // $FlowFixMe
        name: this.dist.name
      } : null,
      compiler: this.compiler ? this.compiler.toString() : null,
      tester: this.tester ? this.tester.toString() : null,
      log: {
        message: this.log.message,
        date: this.log.date,
        username: this.log.username,
        email: this.log.email
      },
      specsResults: this.specsResults,
      docs: this.docs,
      dependencies: this.dependencies.map(dep => dep.toString()),
      flattenedDependencies: this.flattenedDependencies.map(dep => dep.toString()),
      packageDependencies: this.packageDependencies
    }, val => !!val);
  }

  toBuffer() {
    const obj = this.toObject();
    return Buffer.from(JSON.stringify(obj));
  }

  static parse(contents) {
    var _JSON$parse = JSON.parse(contents);

    const impl = _JSON$parse.impl,
          specs = _JSON$parse.specs,
          dist = _JSON$parse.dist,
          compiler = _JSON$parse.compiler,
          tester = _JSON$parse.tester,
          log = _JSON$parse.log,
          docs = _JSON$parse.docs,
          specsResults = _JSON$parse.specsResults,
          dependencies = _JSON$parse.dependencies,
          flattenedDependencies = _JSON$parse.flattenedDependencies,
          packageDependencies = _JSON$parse.packageDependencies;


    return new Version({
      impl: {
        file: (_objects || _load_objects()).Ref.from(impl.file),
        name: impl.name
      },
      specs: specs ? {
        file: (_objects || _load_objects()).Ref.from(specs.file),
        name: specs.name
      } : null,
      dist: dist ? {
        file: (_objects || _load_objects()).Ref.from(dist.file),
        name: dist.name
      } : null,
      compiler: compiler ? (_bitId || _load_bitId()).BitId.parse(compiler) : null,
      tester: tester ? (_bitId || _load_bitId()).BitId.parse(tester) : null,
      log: {
        message: log.message,
        date: log.date,
        username: log.username,
        email: log.email
      },
      specsResults,
      docs,
      dependencies: (_bitId || _load_bitId()).BitIds.deserialize(dependencies),
      flattenedDependencies: (_bitId || _load_bitId()).BitIds.deserialize(flattenedDependencies),
      packageDependencies
    });
  }

  static fromComponent(_ref2) {
    let component = _ref2.component,
        impl = _ref2.impl,
        specs = _ref2.specs,
        dist = _ref2.dist,
        flattenedDeps = _ref2.flattenedDeps,
        message = _ref2.message,
        specsResults = _ref2.specsResults,
        username = _ref2.username,
        email = _ref2.email;

    return new Version({
      impl: {
        file: impl.hash(),
        name: component.implFile
      },
      specs: specs ? {
        file: specs.hash(),
        name: component.specsFile
      } : null,
      dist: dist ? {
        file: dist.hash(),
        name: (_constants || _load_constants()).DEFAULT_BUNDLE_FILENAME
      } : null,
      compiler: component.compilerId,
      tester: component.testerId,
      log: {
        message,
        username,
        email,
        date: Date.now().toString()
      },
      specsResults,
      docs: component.docs,
      packageDependencies: component.packageDependencies,
      flattenedDependencies: flattenedDeps,
      dependencies: component.dependencies
    });
  }
}
exports.default = Version;