'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('../repository'));
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope'));
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../consumer/component'));
}

var _dependencyMaps;

function _load_dependencyMaps() {
  return _dependencyMaps = require('../dependency-maps');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class External extends (_repository || _load_repository()).default {

  constructor(scope, externalMap) {
    super(scope);
    this.externalMap = externalMap;
  }

  getPath() {
    return (_path || _load_path()).default.join(super.getPath(), (_constants || _load_constants()).BIT_EXTERNAL_DIRNAME);
  }

  composePath(bitId) {
    return (_path || _load_path()).default.join(bitId.scope, bitId.box, bitId.name, bitId.version);
  }

  store(bit) {
    return bit.cd(this.composePath(bit.getId())).write(true).then(() => this.externalMap.setBit(bit)).then(() => bit);
  }

  storeMany(bits) {
    return Promise.all(bits.map(bit => bit.store()));
  }

  get(id) {
    return (_component || _load_component()).default.load(this.composePath(id), id.name).then(bit => {
      return { bit, success: true };
    }).catch(() => {
      return { id, success: false };
    });
  }

  write() {
    return this.externalMap.write();
  }
}
exports.default = External;