'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _index;

function _load_index() {
  return _index = require('./index');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class SourcesJson extends Map {

  constructor(sourcesTuples, sources) {
    super(sourcesTuples);
    this.sources = sources;
  }

  getPath() {
    return (_path || _load_path()).default.join(this.sources.getPath(), (_constants || _load_constants()).SOURCES_JSON);
  }

  toPlainObject() {
    const obj = {};

    this.forEach((bitIds, bitId) => {
      obj[bitId.toString()] = bitIds.serialize();
    });

    return obj;
  }

  toJson() {
    return JSON.stringify(this.toPlainObject());
  }

  write() {
    return (0, (_utils || _load_utils()).writeFile)(this.getPath(), this.toJson());
  }

  static load(json, sources) {
    const tuples = [];

    (0, (_utils || _load_utils()).forEach)(json, (bitIds, bitId) => {
      tuples.push([(_bitId || _load_bitId()).BitId.parse(bitId), (_bitId || _load_bitId()).BitIds.deserialize(bitIds)]);
    });

    return new SourcesJson(tuples, sources);
  }
}
exports.default = SourcesJson;