'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objects;

function _load_objects() {
  return _objects = require('../objects');
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('../component-objects'));
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('../exceptions');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../models/component'));
}

var _componentVersion;

function _load_componentVersion() {
  return _componentVersion = _interopRequireDefault(require('../component-version'));
}

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('../models/version'));
}

var _source;

function _load_source() {
  return _source = _interopRequireDefault(require('../models/source'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _consumerComponent;

function _load_consumerComponent() {
  return _consumerComponent = _interopRequireDefault(require('../../consumer/component/consumer-component'));
}

var _globalConfig;

function _load_globalConfig() {
  return _globalConfig = _interopRequireWildcard(require('../../api/consumer/lib/global-config'));
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class SourceRepository {

  constructor(scope) {
    this.scope = scope;
  }

  objects() {
    return this.scope.objects;
  }

  findComponent(component) {
    return this.objects().findOne(component.hash()).catch(() => {
      return null;
    });
  }

  getMany(ids) {
    return Promise.all(ids.map(id => {
      return this.get(id).then(component => {
        return {
          id,
          component
        };
      });
    }));
  }

  get(bitId) {
    const component = (_component || _load_component()).default.fromBitId(bitId);
    return this.findComponent(component);
  }

  getObjects(id) {
    return this.get(id).then(component => {
      if (!component) throw new (_exceptions || _load_exceptions()).ComponentNotFound(id);
      return component.collectObjects(this.objects());
    });
  }

  findOrAddComponent(props) {
    const comp = (_component || _load_component()).default.from(props);
    return this.findComponent(comp).then(component => {
      if (!component) return comp;
      return component;
    });
  }

  addSource(_ref) {
    let source = _ref.source,
        depIds = _ref.depIds,
        message = _ref.message,
        force = _ref.force,
        loader = _ref.loader;

    const objectRepo = this.objects();

    return this.findOrAddComponent(source).then(component => {
      const impl = (_source || _load_source()).default.from(Buffer.from(source.impl.src));
      try {
        source.build(this.scope);
      } catch (e) {
        throw e;
      }

      const dist = source.dist ? (_source || _load_source()).default.from(Buffer.from(source.dist.src)) : null;
      const specs = source.specs ? (_source || _load_source()).default.from(Buffer.from(source.specs.src)) : null;

      const username = (_globalConfig || _load_globalConfig()).getSync((_constants || _load_constants()).USER_NAME_KEY);
      const email = (_globalConfig || _load_globalConfig()).getSync((_constants || _load_constants()).USER_EMAIL_KEY);

      if (loader) {
        loader.text = 'running specs';
      }
      return source.runSpecs(this.scope, !force).then(specsResults => {
        const version = (_version || _load_version()).default.fromComponent({
          component: source,
          impl,
          specs,
          dist,
          flattenedDeps: depIds,
          specsResults,
          message,
          username,
          email
        });

        component.addVersion(version);

        objectRepo.add(version).add(component).add(impl).add(specs).add(dist);

        return component;
      });
    });
  }

  put(_ref2) {
    let component = _ref2.component,
        objects = _ref2.objects;

    const repo = this.objects();
    repo.add(component);
    objects.forEach(obj => repo.add(obj));
    return component;
  }

  clean(bitId) {
    return this.get(bitId).then(component => component.remove(this.objects()));
  }

  merge(_ref3) {
    let component = _ref3.component,
        objects = _ref3.objects;
    let inScope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

    if (inScope) component.scope = this.scope.name;
    return this.findComponent(component).then(existingComponent => {
      if (!existingComponent || component.compatibleWith(existingComponent)) {
        return this.put({ component, objects });
      }

      throw new (_exceptions || _load_exceptions()).MergeConflict();
    });
  }
}
exports.default = SourceRepository;