'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ScopeJson = undefined;
exports.getPath = getPath;

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _remotes;

function _load_remotes() {
  return _remotes = require('../remotes');
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getPath(scopePath) {
  return (_path || _load_path()).default.join(scopePath, (_constants || _load_constants()).SCOPE_JSON);
}
class ScopeJson {

  constructor(_ref) {
    let name = _ref.name,
        remotes = _ref.remotes,
        resolverPath = _ref.resolverPath,
        groupName = _ref.groupName;

    this.name = name;
    this.resolverPath = resolverPath;
    this.remotes = remotes || {};
    this.groupName = groupName || '';
  }

  toPlainObject() {
    return (0, (_utils || _load_utils()).cleanObject)({
      name: this.name,
      remotes: this.remotes,
      resolverPath: this.resolverPath,
      groupName: this.groupName
    });
  }

  toJson() {
    let readable = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : true;

    if (!readable) return JSON.stringify(this.toPlainObject());
    return JSON.stringify(this.toPlainObject(), null, 4);
  }

  addRemote(remote) {
    this.remotes[remote.name] = remote.host;
    return this;
  }

  rmRemote(name) {
    delete this.remotes[name];
    return this;
  }

  write(path) {
    return (0, (_utils || _load_utils()).writeFile)((_path || _load_path()).default.join(path, (_constants || _load_constants()).SCOPE_JSON), this.toJson());
  }

  static loadFromJson(json) {
    return new ScopeJson(JSON.parse(json));
  }
}
exports.ScopeJson = ScopeJson;