'use strict';

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _stream;

function _load_stream() {
  return _stream = require('stream');
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../consumer/component'));
}

var _serverlessIndex;

function _load_serverlessIndex() {
  return _serverlessIndex = _interopRequireDefault(require('./serverless-index'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

let localIndex;

function tokenizeStr(str) {
  return str.trim().split(/(?=[A-Z])/).join(' ').toLowerCase().split(/ |_|-/).join(' ');
}

/**
 * returns the first sentence of the description. 
 * @param {string} desc
 * @return {string}
 */
function minimizeDescription() {
  let desc = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

  return desc.split(/\.|;/)[0]; // split by a dot or a semicolon
}

function prepareDoc(docs, component) {
  const name = component.name;
  const box = component.box;
  const functionNames = docs.map(doc => doc.name).join(' ');
  return {
    id: `${box}_${name}`,
    name,
    box,
    tokenizedName: tokenizeStr(name),
    tokenizedBox: tokenizeStr(box),
    functionNames,
    tokenizedFunctionNames: tokenizeStr(functionNames),
    description: docs.map(doc => doc.description).join(' '),
    minDescription: docs.map(doc => minimizeDescription(doc.description)).join(' ')
  };
}

function addAllToLocalIndex(components) {
  return new Promise((resolve, reject) => {
    const docs = components.map(component => prepareDoc(component.docs, component));
    localIndex.then(indexInstance => {
      const docStream = new (_stream || _load_stream()).Readable({ objectMode: true });
      // $FlowFixMe: a flow bug. Stream can be an object as well when objectMode is true
      docs.map(doc => docStream.push(doc));
      docStream.push(null);
      docStream.pipe(indexInstance.defaultPipeline()).pipe(indexInstance.add()).on('data', d => {
        // this function needs to be called if you want to listen for the end event
      }).on('end', () => {
        resolve('The indexing has been completed');
      });
    });
  });
}

function addToLocalIndex(component) {
  return new Promise((resolve, reject) => {
    const doc = prepareDoc(component.docs, component);
    localIndex.then(indexInstance => {
      const docStream = new (_stream || _load_stream()).Readable({ objectMode: true });
      // $FlowFixMe: a flow bug. Stream can be an object as well when objectMode is true
      docStream.push(doc);
      docStream.push(null);
      docStream.pipe(indexInstance.defaultPipeline()).pipe(indexInstance.add()).on('data', d => {
        // this function needs to be called if you want to listen for the end event
      }).on('end', () => {
        resolve(component);
      });
    });
  });
}

function index(component, scopePath) {
  localIndex = (_serverlessIndex || _load_serverlessIndex()).default.initializeIndex(scopePath);
  return addToLocalIndex(component);
}

function indexAll(path, components) {
  return new Promise((resolve, reject) => {
    if (!components) return reject('The scope is empty');
    (_serverlessIndex || _load_serverlessIndex()).default.deleteDb(path);
    localIndex = (_serverlessIndex || _load_serverlessIndex()).default.initializeIndex(path);
    const results = addAllToLocalIndex(components);
    return resolve(results);
  });
}

module.exports = {
  index,
  indexAll,
  tokenizeStr
};