'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _indexer;

function _load_indexer() {
  return _indexer = _interopRequireDefault(require('./indexer'));
}

var _stopwords;

function _load_stopwords() {
  return _stopwords = _interopRequireDefault(require('./stopwords'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const boost = {
  box: 3,
  tokenizedBox: 2,
  name: 5,
  tokenizedName: 4,
  functionNames: 2,
  tokenizedFunctionNames: 2,
  minDescription: 1
};

function queryItem(field, queryStr) {
  return {
    AND: { [field]: queryStr.toLowerCase().split(' ') },
    BOOST: boost[field]
  };
}

function buildQuery(queryStr) {
  const queryStrWithoutStopwords = queryStr.split(' ').filter(word => !(_stopwords || _load_stopwords()).default.includes(word)).join(' ');
  const tokenizedQuery = (_indexer || _load_indexer()).default.tokenizeStr(queryStr);
  const query = [];
  query.push(queryItem('box', queryStr));
  query.push(queryItem('tokenizedBox', queryStr));
  query.push(queryItem('name', queryStr));
  query.push(queryItem('tokenizedName', tokenizedQuery));
  query.push(queryItem('functionNames', queryStr));
  query.push(queryItem('tokenizedFunctionNames', tokenizedQuery));
  query.push(queryItem('minDescription', queryStrWithoutStopwords));
  return query;
}

exports.default = buildQuery;