'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _child_process;

function _load_child_process() {
  return _child_process = require('child_process');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope/scope'));
}

var _specsResults;

function _load_specsResults() {
  return _specsResults = require('../consumer/specs-results');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function run(_ref) {
  let scope = _ref.scope,
      testerFilePath = _ref.testerFilePath,
      implSrc = _ref.implSrc,
      specsSrc = _ref.specsSrc;

  const implFilePath = scope.tmp.saveSync(implSrc);
  const specsFilePath = scope.tmp.saveSync(specsSrc);

  const removeTmpFiles = () => {
    scope.tmp.removeSync(implFilePath);
    scope.tmp.removeSync(specsFilePath);
  };

  function getDebugPort() {
    const debugPortArgName = '--debug-brk';
    try {
      const execArgv = process.execArgv.map(arg => arg.split('='));
      const execArgvObj = (_ramda || _load_ramda()).default.fromPairs(execArgv);
      if (execArgvObj[debugPortArgName]) return parseInt(execArgvObj[debugPortArgName]);
    } catch (e) {
      return null;
    }

    return null;
  }

  return new Promise((resolve, reject) => {
    const debugPort = getDebugPort();
    const openPort = debugPort ? debugPort + 1 : null;

    const child = (0, (_child_process || _load_child_process()).fork)((_path || _load_path()).default.join(__dirname, 'worker.js'), {
      execArgv: openPort ? [`--debug=${openPort.toString()}`] : [],
      silent: true,
      env: {
        __impl__: implFilePath,
        __specs__: specsFilePath,
        __tester__: testerFilePath
      }
    });

    process.on('exit', () => {
      child.kill('SIGKILL');
    });

    child.on('message', (_ref2) => {
      let type = _ref2.type,
          payload = _ref2.payload;

      removeTmpFiles();
      if (type === 'error') return reject(payload);
      return resolve(payload);
    });

    child.on('error', e => {
      removeTmpFiles();
      reject(e);
    });
  });
}
exports.default = {
  run
};