'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.add = add;
exports.remove = remove;
exports.list = list;
exports.refresh = refresh;

var _scope;

function _load_scope() {
  return _scope = require('../../../scope');
}

var _remotes;

function _load_remotes() {
  return _remotes = require('../../../remotes');
}

var _globalConfig;

function _load_globalConfig() {
  return _globalConfig = require('../../../global-config');
}

function buildRemote(url) {
  return new (_remotes || _load_remotes()).Remote(url);
}
function add(url, global) {
  const remote = buildRemote(url);
  return remote.scope().then(scopeDesc => {
    remote.name = scopeDesc.name;

    if (global) {
      return (_globalConfig || _load_globalConfig()).GlobalRemotes.load().then(globalRemotes => {
        return globalRemotes.addRemote(remote).write().then(() => remote);
      });
    }

    return (0, (_scope || _load_scope()).loadScope)().then(scope => {
      return scope.scopeJson.addRemote(remote).write(scope.getPath()).then(() => remote);
    });
  });
}

function remove(name, global) {
  if (global) {
    return (_globalConfig || _load_globalConfig()).GlobalRemotes.load().then(globalRemotes => {
      return globalRemotes.rmRemote(name).write().then(() => name);
    });
  }

  return (0, (_scope || _load_scope()).loadScope)().then(scope => {
    return scope.scopeJson.rmRemote(name).write(scope.getPath()).then(() => name);
  });
}

function list(global) {
  if (global) {
    return (_globalConfig || _load_globalConfig()).GlobalRemotes.load().then(globalRemotes => globalRemotes.toPlainObject());
  }

  return (0, (_scope || _load_scope()).loadScope)().then(scope => {
    return scope.remotes().then(remotes => remotes.toPlainObject());
  });
}

function refresh() {}
//# sourceMappingURL=remote.js.map