'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('../version'));
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class BitId {

  constructor(_ref) {
    let scope = _ref.scope,
        box = _ref.box,
        name = _ref.name,
        version = _ref.version;

    this.scope = scope;
    this.box = box || 'global';
    this.name = name;
    this.version = version;
  }

  changeScope(newScope) {
    this.scope = newScope;
    return this;
  }

  getScopeWithoutRemoteAnnotaion() {
    return this.scope.replace((_constants || _load_constants()).REMOTE_ALIAS_SIGN, '');
  }

  isLocal(scopeName) {
    return scopeName === this.getScopeWithoutRemoteAnnotaion();
  }

  getVersion() {
    return (_version || _load_version()).default.parse(this.version);
  }

  toString() {
    let ignoreScope = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    const name = this.name,
          box = this.box,
          version = this.version;

    const scope = this.scope;

    if (ignoreScope) return [box, name].join('/').concat(`::${version}`);
    return [scope, box, name].join('/').concat(`::${version}`);
  }

  toObject() {
    const key = [this.scope, this.box, this.name].join('/');
    const value = this.version;

    return { [key]: value };
  }

  static parse(id, realScopeName) {
    let version = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : (_constants || _load_constants()).LATEST_BIT_VERSION;

    if (!id || id === (_constants || _load_constants()).NO_PLUGIN_TYPE) {
      return null;
    }
    if ((0, (_utils || _load_utils()).contains)(id, (_constants || _load_constants()).VERSION_DELIMITER)) {
      var _id$split = id.split((_constants || _load_constants()).VERSION_DELIMITER),
          _id$split2 = _slicedToArray(_id$split, 2);

      const newId = _id$split2[0],
            newVersion = _id$split2[1];

      id = newId;
      version = newVersion;
    }

    const splited = id.split('/');
    if (splited.length === 3) {
      var _splited = _slicedToArray(splited, 3);

      const scope = _splited[0],
            box = _splited[1],
            name = _splited[2];

      if (scope === (_constants || _load_constants()).LOCAL_SCOPE_NOTATION && !realScopeName) {
        throw new Error('real scope name is required in bitId.parse with @this notation');
      }

      // $FlowFixMe (in this case the realScopeName is not null)
      return new BitId({
        scope: scope === (_constants || _load_constants()).LOCAL_SCOPE_NOTATION ? realScopeName : scope,
        box,
        name,
        version
      });
    }

    if (splited.length === 2) {
      var _splited2 = _slicedToArray(splited, 2);

      const scope = _splited2[0],
            name = _splited2[1];

      if (scope === (_constants || _load_constants()).LOCAL_SCOPE_NOTATION && !realScopeName) {
        throw new Error('real scope name is required in bitId.parse with @this notation');
      }

      // $FlowFixMe (in this case the realScopeName is not null)
      return new BitId({
        scope: scope === (_constants || _load_constants()).LOCAL_SCOPE_NOTATION ? realScopeName : scope,
        name,
        version
      });
    }

    throw new (_exceptions || _load_exceptions()).InvalidBitId();
  }
}
exports.default = BitId;
//# sourceMappingURL=bit-id.js.map