'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ora;

function _load_ora() {
  return _ora = _interopRequireDefault(require('ora'));
}

var _commander;

function _load_commander() {
  return _commander = _interopRequireDefault(require('commander'));
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _defaultErrorHandler;

function _load_defaultErrorHandler() {
  return _defaultErrorHandler = _interopRequireDefault(require('./default-error-handler'));
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const chalk = require('chalk');

function logAndExit(msg) {
  console.log(msg); // eslint-disable-line
  process.exit();
}

function logErrAndExit(msg) {
  console.error(msg); // eslint-disable-line
  process.exit(1);
}

function parseSubcommandFromArgs(args) {
  if (typeof (0, (_utils || _load_utils()).first)(args) === 'string') return (0, (_utils || _load_utils()).first)(args);
  return null;
}

function parseCommandName(commandName) {
  if (!commandName) return '';
  return (0, (_utils || _load_utils()).first)(commandName.split(' '));
}

function getOpts(c, opts) {
  const options = {};

  opts.forEach((_ref) => {
    var _ref2 = _slicedToArray(_ref, 2);

    let name = _ref2[1];

    name = parseCommandName(name);
    options[name] = c[name];
  });

  return options;
}

function execAction(command, concrete, args) {
  // $FlowFixMe
  const opts = getOpts(concrete, command.opts);
  if (command.loader) {
    const autoStart = command.loader.autoStart || true;
    command.loader = (0, (_ora || _load_ora()).default)({ spinner: (_constants || _load_constants()).SPINNER_TYPE, text: command.loader.text || '' });
    if (autoStart) {
      command.loader.start();
    }
  }

  command.action(args.slice(0, args.length - 1), opts).then(data => {
    if (command.loader) {
      command.loader.stop();
    }
    return logAndExit(command.report(data));
  }).catch(err => {
    if (command.loader) {
      command.loader.stop();
    }
    const errorHandled = (0, (_defaultErrorHandler || _load_defaultErrorHandler()).default)(err) || command.handleError(err);

    if (!command.private && errorHandled) logAndExit(errorHandled);else logErrAndExit(err);
  });
}

// @TODO add help for subcommands
function registerAction(command, concrete) {
  concrete.action(function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (!(0, (_utils || _load_utils()).empty)(command.commands)) {
      // $FlowFixMe
      const subcommandName = parseSubcommandFromArgs(args);
      const subcommand = command.commands.find(cmd => {
        return subcommandName === (parseCommandName(cmd.name) || cmd.alias);
      });

      args.shift();
      if (subcommand) return execAction(subcommand, concrete, args);
    }

    return execAction(command, concrete, args);
  });
}

class CommandRegistrar {

  registerBaseCommand() {
    (_commander || _load_commander()).default.version(this.version).usage(this.usage).description(this.description);
  }

  constructor(usage, description, version, commands) {
    this.usage = usage;
    this.description = description;
    this.version = version;
    this.commands = commands;
  }

  registerCommands() {
    function createOptStr(alias, name) {
      return `-${alias}, --${name}`;
    }

    function register(command, commanderCmd) {
      // $FlowFixMe
      const concrete = commanderCmd.command(command.name, null, { noHelp: command.private }).description(command.description).alias(command.alias);

      command.opts.forEach((_ref3) => {
        var _ref4 = _slicedToArray(_ref3, 3);

        let alias = _ref4[0],
            name = _ref4[1],
            description = _ref4[2];

        concrete.option(createOptStr(alias, name), description);
      });

      command.commands.forEach(nestedCmd => {
        register(nestedCmd, concrete);
      });

      return registerAction(command, concrete);
    }

    this.commands.forEach(cmd => register(cmd, (_commander || _load_commander()).default));
  }

  outputHelp() {
    const args = process.argv.slice(2);
    if (!args.length) {
      (_commander || _load_commander()).default.help();
      return this;
    }

    const subcommand = args[0];
    const cmdList = this.commands.map(cmd => (0, (_utils || _load_utils()).first)(cmd.name.split(' ')));
    const aliasList = this.commands.map(cmd => (0, (_utils || _load_utils()).first)(cmd.alias.split(' ')));

    if (cmdList.indexOf(subcommand) === -1 && aliasList.indexOf(subcommand) === -1) {
      console.log(chalk.yellow(`warning: no command named '${chalk.bold(subcommand)}' was found...\nsee 'bit --help' for additional information.`));
      return this;
    }

    return this;
  }

  errorHandler(err) {
    console.error(err);
  }

  run() {
    this.registerBaseCommand();
    this.registerCommands();
    (_commander || _load_commander()).default.parse(process.argv);
    this.outputHelp();

    return this;
  }
}
exports.default = CommandRegistrar;
//# sourceMappingURL=command-registrar.js.map