'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RELEASE_SERVER = exports.BIT_INSTALL_METHOD = exports.BIT_VERSION = exports.DEFAULT_RESOLVER = exports.SCOPE_JSON = exports.LATEST_TESTED_MARK = exports.BIT_CACHE_DIRNAME = exports.BIT_TMP_DIRNAME = exports.BIT_SOURCES_DIRNAME = exports.DEPENDENCY_DELIMITER = exports.LATEST = exports.BIT_REGISTRY = exports.AUTH_CACHE_FILENAME = exports.MODULES_CACHE_FILENAME = exports.BIT_CONTAINER_FOLDERS = exports.BIT_HIDDEN_DIR = exports.GLOBAL_BIT_CACHE = exports.GLOBAL_REMOTES = exports.SOURCES_MAP = exports.EXTERNAL_MAP = exports.SOURCES_JSON = exports.GLOBAL_CONFIG_FILE = exports.GLOBAL_CONFIG = exports.APP_CACHE_DIR = exports.MODULES_CACHE_DIR = exports.PLUGINS_DIR = exports.CACHE_ROOT = exports.SPINNER_TYPE = exports.USER_NAME_KEY = exports.USER_EMAIL_KEY = exports.DEFAULT_DEPENDENCIES = exports.DEFAULT_REMOTES = exports.LOCAL_SCOPE_NOTATION = exports.BIT_EXTERNAL_DIRNAME = exports.DEPENDENCY_MAP_FILENAME = exports.VERSION_DELIMITER = exports.DEFAULT_BOX_NAME = exports.SPACE_DELIMITER = exports.NULL_BYTE = exports.OBJECTS_DIR = exports.LATEST_BIT_VERSION = exports.DEFAULT_BIT_VERSION = exports.DEFAULT_BUNDLE_FILENAME = exports.DEFAULT_DIST_DIRNAME = exports.DEFAULT_TESTER_ID = exports.DEFAULT_COMPILER_ID = exports.NO_PLUGIN_TYPE = exports.DEFAULT_SPECS_NAME = exports.DEFAULT_IMPL_NAME = exports.REMOTE_ALIAS_SIGN = exports.BIT_JSON = exports.INLINE_BITS_DIRNAME = exports.BITS_DIRNAME = exports.BIT_USAGE = exports.BIT_DESCRIPTION = exports.RESOURCES = undefined;

var _cliSpinners;

function _load_cliSpinners() {
  return _cliSpinners = _interopRequireDefault(require('cli-spinners'));
}

var _os;

function _load_os() {
  return _os = _interopRequireDefault(require('os'));
}

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const userHome = require('user-home');

const packageFile = require('../package.json');

const isWindows = (_os || _load_os()).default.platform() === 'win32';

function getDirectory() {
  if (process.platform === 'win32' && process.env.LOCALAPPDATA) {
    return (_path || _load_path()).default.join(process.env.LOCALAPPDATA, 'Bit');
  }

  return (_path || _load_path()).default.join(userHome, '.bit');
}

function getCacheDirectory() {
  if (process.platform === 'darwin') {
    return (_path || _load_path()).default.join(userHome, 'Library', 'Caches', 'Bit');
  }

  return getDirectory();
}

const RESOURCES = exports.RESOURCES = (_path || _load_path()).default.resolve((_path || _load_path()).default.join(__dirname, '../resources'));

const BIT_DESCRIPTION = exports.BIT_DESCRIPTION = 'bit is a free and open source tool designed for easy use, maintainance and discovery of code components.';

const BIT_USAGE = exports.BIT_USAGE = '[--version] [--help] <command> [<args>]';

const BITS_DIRNAME = exports.BITS_DIRNAME = 'components';

const INLINE_BITS_DIRNAME = exports.INLINE_BITS_DIRNAME = 'inline_components';

const BIT_JSON = exports.BIT_JSON = 'bit.json';

const REMOTE_ALIAS_SIGN = exports.REMOTE_ALIAS_SIGN = '@';

const DEFAULT_IMPL_NAME = exports.DEFAULT_IMPL_NAME = 'impl.js';

const DEFAULT_SPECS_NAME = exports.DEFAULT_SPECS_NAME = 'spec.js';

const NO_PLUGIN_TYPE = exports.NO_PLUGIN_TYPE = 'none';

const DEFAULT_COMPILER_ID = exports.DEFAULT_COMPILER_ID = NO_PLUGIN_TYPE;

const DEFAULT_TESTER_ID = exports.DEFAULT_TESTER_ID = NO_PLUGIN_TYPE;

const DEFAULT_DIST_DIRNAME = exports.DEFAULT_DIST_DIRNAME = 'dist';

const DEFAULT_BUNDLE_FILENAME = exports.DEFAULT_BUNDLE_FILENAME = 'dist.js';

const DEFAULT_BIT_VERSION = exports.DEFAULT_BIT_VERSION = 1;

const LATEST_BIT_VERSION = exports.LATEST_BIT_VERSION = 'latest';

const OBJECTS_DIR = exports.OBJECTS_DIR = 'objects';

const NULL_BYTE = exports.NULL_BYTE = '\u0000';

const SPACE_DELIMITER = exports.SPACE_DELIMITER = ' ';

const DEFAULT_BOX_NAME = exports.DEFAULT_BOX_NAME = 'global';

const VERSION_DELIMITER = exports.VERSION_DELIMITER = '::';

const DEPENDENCY_MAP_FILENAME = exports.DEPENDENCY_MAP_FILENAME = 'dependencies.json';

const BIT_EXTERNAL_DIRNAME = exports.BIT_EXTERNAL_DIRNAME = 'external';

const LOCAL_SCOPE_NOTATION = exports.LOCAL_SCOPE_NOTATION = '@this';

const DEFAULT_REMOTES = exports.DEFAULT_REMOTES = {};

const DEFAULT_DEPENDENCIES = exports.DEFAULT_DEPENDENCIES = {};

const USER_EMAIL_KEY = exports.USER_EMAIL_KEY = 'user.email';

const USER_NAME_KEY = exports.USER_NAME_KEY = 'user.name';

const SPINNER_TYPE = exports.SPINNER_TYPE = isWindows ? (_cliSpinners || _load_cliSpinners()).default.line : (_cliSpinners || _load_cliSpinners()).default.dots12;

/**
 * cache root directory
 */
const CACHE_ROOT = exports.CACHE_ROOT = getCacheDirectory();

/**
 * global plugins directory
 */
const PLUGINS_DIR = exports.PLUGINS_DIR = (_path || _load_path()).default.join(CACHE_ROOT, 'plugins');

/**
 * modules cache directory
 */
const MODULES_CACHE_DIR = exports.MODULES_CACHE_DIR = (_path || _load_path()).default.join(CACHE_ROOT, 'modules');

/**
 * app cache directory
 */
const APP_CACHE_DIR = exports.APP_CACHE_DIR = (_path || _load_path()).default.join(CACHE_ROOT, 'app');

/**
 * glboal config directories
 */
const GLOBAL_CONFIG = exports.GLOBAL_CONFIG = (_path || _load_path()).default.join(CACHE_ROOT, 'config');

const GLOBAL_CONFIG_FILE = exports.GLOBAL_CONFIG_FILE = 'config.json';

const SOURCES_JSON = exports.SOURCES_JSON = 'sources.json';

const EXTERNAL_MAP = exports.EXTERNAL_MAP = 'externals.json';

const SOURCES_MAP = exports.SOURCES_MAP = 'sources.json';

const GLOBAL_REMOTES = exports.GLOBAL_REMOTES = 'global-remotes.json';

/**
 * bits cache directory 
 */
const GLOBAL_BIT_CACHE = exports.GLOBAL_BIT_CACHE = (_path || _load_path()).default.join(CACHE_ROOT, 'bits');

const BIT_HIDDEN_DIR = exports.BIT_HIDDEN_DIR = '.bit';

const BIT_CONTAINER_FOLDERS = exports.BIT_CONTAINER_FOLDERS = ['sources'];

/**
 * modules cache filename
 */
const MODULES_CACHE_FILENAME = exports.MODULES_CACHE_FILENAME = (_path || _load_path()).default.join(MODULES_CACHE_DIR, '.roadrunner.json');

/**
 * auth cache fileName
 */
const AUTH_CACHE_FILENAME = exports.AUTH_CACHE_FILENAME = (_path || _load_path()).default.join(CACHE_ROOT, '.auth.json');

/**
 * bit registry default URL.
 */
const BIT_REGISTRY = exports.BIT_REGISTRY = '';

const LATEST = exports.LATEST = 'latest';

const DEPENDENCY_DELIMITER = exports.DEPENDENCY_DELIMITER = '/';

const BIT_SOURCES_DIRNAME = exports.BIT_SOURCES_DIRNAME = 'source';

const BIT_TMP_DIRNAME = exports.BIT_TMP_DIRNAME = 'tmp';

const BIT_CACHE_DIRNAME = exports.BIT_CACHE_DIRNAME = 'cache';

const LATEST_TESTED_MARK = exports.LATEST_TESTED_MARK = '*';

const SCOPE_JSON = exports.SCOPE_JSON = 'scope.json';

const DEFAULT_RESOLVER = exports.DEFAULT_RESOLVER = () => '';

/**
 * current bit application version
 */
const BIT_VERSION = exports.BIT_VERSION = packageFile.version;

const BIT_INSTALL_METHOD = exports.BIT_INSTALL_METHOD = packageFile.installationMethod;

const RELEASE_SERVER = exports.RELEASE_SERVER = 'https://api-stg.bitsrc.io/release';
//# sourceMappingURL=constants.js.map