'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs;

function _load_fs() {
  return _fs = _interopRequireDefault(require('fs'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _abstractBitJson;

function _load_abstractBitJson() {
  return _abstractBitJson = _interopRequireDefault(require('./abstract-bit-json'));
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function composePath(bitPath) {
  return (_path || _load_path()).default.join(bitPath, (_constants || _load_constants()).BIT_JSON);
}

function hasExisting(bitPath) {
  return (_fs || _load_fs()).default.existsSync(composePath(bitPath));
}

class ConsumerBitJson extends (_abstractBitJson || _load_abstractBitJson()).default {

  write(_ref) {
    let bitDir = _ref.bitDir;
    var _ref$override = _ref.override;
    let override = _ref$override === undefined ? true : _ref$override;

    return new Promise((resolve, reject) => {
      if (!override && hasExisting(bitDir)) {
        throw new (_exceptions || _load_exceptions()).BitJsonAlreadyExists();
      }

      const repspond = (err, res) => {
        if (err) return reject(err);
        return resolve(res);
      };

      (_fs || _load_fs()).default.writeFile(composePath(bitDir), this.toJson(), repspond);
    });
  }

  static create() {
    return new ConsumerBitJson({});
  }

  static ensure(dirPath) {
    return new Promise(resolve => {
      return this.load(dirPath).then(resolve).catch(() => resolve(this.create()));
    });
  }

  static fromPlainObject(object) {
    const sources = object.sources,
          env = object.env,
          dependencies = object.dependencies;

    return new ConsumerBitJson({
      impl: (_ramda || _load_ramda()).default.prop('impl', sources),
      spec: (_ramda || _load_ramda()).default.prop('spec', sources),
      compiler: (_ramda || _load_ramda()).default.prop('compiler', env),
      tester: (_ramda || _load_ramda()).default.prop('tester', env),
      dependencies
    });
  }

  static load(dirPath) {
    return new Promise((resolve, reject) => {
      if (!hasExisting(dirPath)) return reject(new (_exceptions || _load_exceptions()).BitJsonNotFound());
      return (_fs || _load_fs()).default.readFile(composePath(dirPath), (err, data) => {
        if (err) return reject(err);
        const file = JSON.parse(data.toString('utf8'));
        return resolve(this.fromPlainObject(file));
      });
    });
  }
}
exports.default = ConsumerBitJson;
//# sourceMappingURL=consumer-bit-json.js.map