'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = parse;

var _doctrine;

function _load_doctrine() {
  return _doctrine = _interopRequireDefault(require('doctrine'));
}

var _exampleTagParser;

function _load_exampleTagParser() {
  return _exampleTagParser = _interopRequireDefault(require('./example-tag-parser'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/** @Flow */
function formatTag(tag) {
  delete tag.title;
  if (!tag.type) return tag;
  let formattedType = (_doctrine || _load_doctrine()).default.type.stringify(tag.type);
  if (tag.type.type === (_doctrine || _load_doctrine()).default.type.Syntax.TypeApplication) {
    // Doctrine adds a dot after the generic type for historical reasons.
    // see here for more info: https://github.com/eslint/doctrine/issues/185
    formattedType = formattedType.replace('.<', '<');
  }
  tag.type = formattedType;

  return tag;
}

function extractDataRegex(doc, doclets) {
  const commentsAst = (_doctrine || _load_doctrine()).default.parse(doc.trim(), { unwrap: true, recoverable: true });
  if (!commentsAst) return;

  const args = [];
  const description = commentsAst.description;
  let returns = {};
  let isStatic = false;
  let access = 'public';
  let examples = [];
  let name = '';

  for (const tag of commentsAst.tags) {
    switch (tag.title) {
      case 'name':
        name = tag.name;
        break;
      case 'param':
      case 'arg':
      case 'argument':
        args.push(formatTag(tag));
        break;
      case 'returns':
      case 'return':
        returns = formatTag(tag);
        break;
      case 'static':
        isStatic = true;
        break;
      case 'private':
      case 'protected':
        access = tag.title;
        break;
      case 'access':
        access = tag.access;
        break;
      case 'example':
        examples.push((0, (_exampleTagParser || _load_exampleTagParser()).default)(tag.description));
        break;
    }
  }

  const doclet = {
    name, // todo: find the function/method name by regex
    description,
    args,
    returns,
    access,
    examples,
    static: isStatic
  };
  doclets.push(doclet);
}

function parse(data) {
  const doclets = [];
  try {
    const jsdocRegex = /[ \t]*\/\*\*\s*\n([^*]*(\*[^/])?)*\*\//g;
    const docs = data.match(jsdocRegex);
    docs.map(doc => extractDataRegex(doc, doclets));
  } catch (e) {
    // never mind, ignore the doc of this source
  }

  return doclets;
}
//# sourceMappingURL=parser.js.map