'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _ramda;

function _load_ramda() {
  return _ramda = require('ramda');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _remote;

function _load_remote() {
  return _remote = _interopRequireDefault(require('./remote'));
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('../scope/component-objects'));
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _remoteResolver;

function _load_remoteResolver() {
  return _remoteResolver = _interopRequireDefault(require('./remote-resolver/remote-resolver'));
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope/scope'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Remotes extends Map {
  constructor() {
    let remotes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    super(remotes);
  }

  validate() {
    const primary = this.values.filter(remote => remote.primary);
    if (primary.length > 1) throw new (_exceptions || _load_exceptions()).PrimaryOverloaded();
    return this.forEach(remote => remote.validate());
  }

  get(name) {
    const remote = super.get(name);
    if (!remote) throw new (_exceptions || _load_exceptions()).RemoteNotFound(name);
    return remote;
  }

  resolve(scopeName, thisScope) {
    if (scopeName.startsWith((_constants || _load_constants()).REMOTE_ALIAS_SIGN)) {
      return Promise.resolve(this.get(scopeName.replace((_constants || _load_constants()).REMOTE_ALIAS_SIGN, '')));
    }

    return (0, (_remoteResolver || _load_remoteResolver()).default)(scopeName, thisScope).then(scopeHost => {
      return new (_remote || _load_remote()).default(scopeHost, scopeName);
    });
  }

  fetch(ids, thisScope) {
    let withoutDeps = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    const byScope = (0, (_ramda || _load_ramda()).groupBy)((0, (_ramda || _load_ramda()).prop)('scope'));
    const promises = [];
    (0, (_utils || _load_utils()).forEach)(byScope(ids), (scopeIds, scopeName) => {
      if (!withoutDeps) {
        promises.push(this.resolve(scopeName, thisScope).then(remote => remote.fetch(scopeIds)));
      } else {
        promises.push(this.resolve(scopeName, thisScope).then(remote => remote.fetchOnes(scopeIds)));
      }
    });

    return Promise.all(promises).then(bits => (0, (_utils || _load_utils()).flatten)(bits));
  }

  toPlainObject() {
    const object = {};

    this.forEach(remote => {
      let name = remote.name;
      if (remote.primary) name = (0, (_utils || _load_utils()).prependBang)(remote.name);
      object[name] = remote.host;
    });

    return object;
  }

  static load(remotes) {
    const models = [];

    if (!remotes) return new Remotes();

    (0, (_utils || _load_utils()).forEach)(remotes, (name, host) => {
      const remote = (_remote || _load_remote()).default.load(name, host);
      models.push([remote.name, remote]);
    });

    return new Remotes(models);
  }
}
exports.default = Remotes;
//# sourceMappingURL=remotes.js.map