'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('./models/component'));
}

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('./models/version'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('./scope'));
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('./objects/repository'));
}

var _versionDependencies;

function _load_versionDependencies() {
  return _versionDependencies = _interopRequireDefault(require('./version-dependencies'));
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('./component-objects'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class ComponentVersion {

  constructor(component, version) {
    this.component = component;
    this.version = version;
  }

  getVersion(repository) {
    return this.component.loadVersion(this.version, repository);
  }

  flattenedDependencies(repository) {
    return this.getVersion(repository).then(version => version.flattenedDependencies);
  }

  toId() {
    return new (_bitId || _load_bitId()).BitId({
      scope: this.component.scope,
      box: this.component.box,
      name: this.component.name,
      version: this.version.toString()
    });
  }

  get id() {
    return this.toId();
  }

  toVersionDependencies(scope, source) {
    return this.getVersion(scope.objects).then(version => {
      if (!version) {
        return scope.remotes().then(remotes => {
          const src = this.id;
          src.scope = source;
          return scope.getExternal(src, remotes, false);
        });
      }

      return version.collectDependencies(scope).then(dependencies => new (_versionDependencies || _load_versionDependencies()).default(this, dependencies, source));
    });
  }

  toConsumer(repo) {
    return this.component.toConsumerComponent(this.version.toString(), this.component.scope, repo);
  }

  toObjects(repo) {
    return this.getVersion(repo).then(version => Promise.all([this.component.asRaw(repo), version.collectRaw(repo), version.asRaw(repo)])).then((_ref) => {
      var _ref2 = _slicedToArray(_ref, 3);

      let compObject = _ref2[0],
          objects = _ref2[1],
          version = _ref2[2];
      return new (_componentObjects || _load_componentObjects()).default(compObject, objects.concat([version]));
    });
  }
}
exports.default = ComponentVersion;
//# sourceMappingURL=component-version.js.map