'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _ramda;

function _load_ramda() {
  return _ramda = require('ramda');
}

var _objects;

function _load_objects() {
  return _objects = require('../objects');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('../exceptions');
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('./version'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _bitId;

function _load_bitId() {
  return _bitId = _interopRequireDefault(require('../../bit-id/bit-id'));
}

var _version2;

function _load_version2() {
  return _version2 = _interopRequireDefault(require('../../version'));
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('../../consumer/component'));
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../scope'));
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('../objects/repository'));
}

var _componentVersion;

function _load_componentVersion() {
  return _componentVersion = _interopRequireDefault(require('../component-version'));
}

var _sources;

function _load_sources() {
  return _sources = require('../../consumer/component/sources');
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('../component-objects'));
}

var _specsResults;

function _load_specsResults() {
  return _specsResults = _interopRequireDefault(require('../../consumer/specs-results'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Component extends (_objects || _load_objects()).BitObject {

  constructor(props) {
    super();
    this.scope = props.scope;
    this.name = props.name;
    this.box = props.box || (_constants || _load_constants()).DEFAULT_BOX_NAME;
    this.versions = props.versions || {};
  }

  get versionArray() {
    return (0, (_utils || _load_utils()).values)(this.versions);
  }

  listVersions() {
    return Object.keys(this.versions).map(versionStr => parseInt(versionStr));
  }

  compatibleWith(component) {
    const differnece = (0, (_utils || _load_utils()).diff)(Object.keys(this.versions), Object.keys(component.versions));
    const comparableObject = (0, (_utils || _load_utils()).filterObject)(this.versions, (val, key) => !differnece.includes(key));
    return (0, (_ramda || _load_ramda()).equals)(component.versions, comparableObject);
  }

  latest() {
    if ((0, (_utils || _load_utils()).empty)(this.versions)) return 0;
    return Math.max(...this.listVersions());
  }

  collectLogs(repo) {
    return repo.findMany(this.versionArray).then(versions => {
      const indexedLogs = (0, (_ramda || _load_ramda()).fromPairs)((0, (_ramda || _load_ramda()).zip)((0, (_ramda || _load_ramda()).keys)(this.versions), (0, (_ramda || _load_ramda()).map)((0, (_ramda || _load_ramda()).prop)('log'), versions)));
      const indexedHashes = (0, (_ramda || _load_ramda()).mapObjIndexed)(ref => (0, (_ramda || _load_ramda()).objOf)('hash', ref.toString()), this.versions);
      return (0, (_ramda || _load_ramda()).mergeWith)((_ramda || _load_ramda()).merge, indexedLogs, indexedHashes);
    });
  }

  collectVersions(repo) {
    return Promise.all(this.listVersions().map(versionNum => {
      return this.toConsumerComponent(String(versionNum), this.scope, repo);
    }));
  }

  addVersion(version) {
    this.versions[this.version()] = version.hash();
    return this;
  }

  version() {
    const latest = this.latest();
    if (latest) return latest + 1;
    return 1;
  }

  id() {
    return [this.scope, this.box, this.name].join('/');
  }

  toObject() {
    function versions(vers) {
      const obj = {};
      (0, (_utils || _load_utils()).forEach)(vers, (ref, version) => {
        obj[version] = ref.toString();
      });
      return obj;
    }

    return {
      box: this.box,
      name: this.name,
      scope: this.scope,
      versions: versions(this.versions)
    };
  }

  loadVersion(version, repository) {
    const versionRef = this.versions[version];
    if (!versionRef) throw new (_exceptions || _load_exceptions()).VersionNotFound();
    return versionRef.load(repository);
  }

  collectObjects(repo) {
    return Promise.all([this.asRaw(repo), this.collectRaw(repo)]).then((_ref) => {
      var _ref2 = _slicedToArray(_ref, 2);

      let rawComponent = _ref2[0],
          objects = _ref2[1];
      return new (_componentObjects || _load_componentObjects()).default(rawComponent, objects);
    });
  }

  remove(repo) {
    const objectRefs = this.versionArray;
    return repo.removeMany(objectRefs.concat([this.hash()]));
  }

  toComponentVersion(versionStr, scopeName) {
    const versionNum = (_version2 || _load_version2()).default.parse(versionStr).resolve(this.listVersions());

    if (!this.versions[versionNum]) throw new Error();
    return new (_componentVersion || _load_componentVersion()).default(this, versionNum, scopeName);
  }

  toConsumerComponent(versionStr, scopeName, repository) {
    const componentVersion = this.toComponentVersion(versionStr, scopeName);
    return componentVersion.getVersion(repository).then(version => {
      const implP = version.impl.file.load(repository);
      const specsP = version.specs ? version.specs.file.load(repository) : null;
      const distP = version.dist ? version.dist.file.load(repository) : null;
      return Promise.all([implP, specsP, distP]).then((_ref3) => {
        var _ref4 = _slicedToArray(_ref3, 3);

        let impl = _ref4[0],
            specs = _ref4[1],
            dist = _ref4[2];

        return new (_component || _load_component()).default({
          name: this.name,
          box: this.box,
          version: componentVersion.version,
          scope: this.scope,
          implFile: version.impl.name,
          specsFile: version.specs ? version.specs.name : null,
          compilerId: version.compiler,
          testerId: version.tester,
          dependencies: version.dependencies,
          packageDependencies: version.packageDependencies,
          impl: new (_sources || _load_sources()).Impl(impl.toString()),
          specs: specs ? new (_sources || _load_sources()).Specs(specs.toString()) : null,
          docs: version.docs,
          dist: dist ? new (_sources || _load_sources()).Dist(dist.toString()) : null,
          specsResults: version.specsResults ? (_specsResults || _load_specsResults()).default.deserialize(version.specsResults) : null
        });
      });
    });
  }

  refs() {
    return (0, (_utils || _load_utils()).values)(this.versions);
  }

  toBuffer() {
    return new Buffer(JSON.stringify(this.toObject()));
  }

  toVersionDependencies(version, scope, source) {
    const versionComp = this.toComponentVersion(version, scope.name);
    return versionComp.toVersionDependencies(scope, source);
  }

  static parse(contents) {
    const rawComponent = JSON.parse(contents);
    return Component.from({
      name: rawComponent.name,
      box: rawComponent.box,
      scope: rawComponent.scope,
      versions: (0, (_utils || _load_utils()).mapObject)(rawComponent.versions, val => (_objects || _load_objects()).Ref.from(val))
    });
  }

  static from(props) {
    return new Component(props);
  }

  static fromBitId(bitId) {
    return new Component({
      name: bitId.name,
      box: bitId.box,
      scope: bitId.getScopeWithoutRemoteAnnotaion()
    });
  }
}
exports.default = Component;
//# sourceMappingURL=component.js.map