'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _scopeLoader;

function _load_scopeLoader() {
  return _scopeLoader = _interopRequireDefault(require('../../scope-loader'));
}

var _scope;

function _load_scope() {
  return _scope = _interopRequireDefault(require('../../scope'));
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('../../component-objects'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../bit-id');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('../exceptions');
}

var _utils;

function _load_utils() {
  return _utils = require('../../../utils');
}

var _search;

function _load_search() {
  return _search = require('../../../search');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Fs {

  constructor(scopePath) {
    this.scopePath = scopePath;
  }

  close() {
    return this;
  }

  getScope() {
    if (!this.scope) throw new (_exceptions || _load_exceptions()).FsScopeNotLoaded();
    return this.scope;
  }

  describeScope() {
    return Promise.resolve(this.getScope().describe());
  }

  push(componentObjects) {
    return this.getScope().export(componentObjects);
  }

  fetch(bitIds) {
    return this.getScope().getObjects(bitIds).then(bitsMatrix => (0, (_utils || _load_utils()).flatten)(bitsMatrix));
  }

  fetchAll(ids) {
    return this.getScope().getObjects(ids);
  }

  fetchOnes(bitIds) {
    return this.getScope().manyOneObjects(bitIds);
  }

  list() {
    return this.getScope().list();
  }

  search(query, reindex) {
    return (_search || _load_search()).searchAdapter.scopeSearch(this.scopePath, query, reindex);
  }

  show(bitId) {
    return this.getScope().loadComponent(bitId);
  }

  connect() {
    return (0, (_scopeLoader || _load_scopeLoader()).default)(this.scopePath).then(scope => {
      this.scope = scope;
      return this;
    });
  }
}
exports.default = Fs;
//# sourceMappingURL=fs.js.map