'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _bitJs;

function _load_bitJs() {
  return _bitJs = _interopRequireDefault(require('bit-js'));
}

var _zlib;

function _load_zlib() {
  return _zlib = require('zlib');
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('./repository'));
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _ref;

function _load_ref() {
  return _ref = _interopRequireDefault(require('./ref'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const sha1 = (0, (_bitJs || _load_bitJs()).default)('sha1');

function parse(buffer, types) {
  var _buffer$toString$spli = buffer.toString().split((_constants || _load_constants()).NULL_BYTE),
      _buffer$toString$spli2 = _slicedToArray(_buffer$toString$spli, 2);

  const headers = _buffer$toString$spli2[0],
        contents = _buffer$toString$spli2[1];

  var _headers$split = headers.split((_constants || _load_constants()).SPACE_DELIMITER),
      _headers$split2 = _slicedToArray(_headers$split, 1);

  const type = _headers$split2[0];

  return types[type].parse(contents);
}

class BitObject {
  id() {
    throw new Error('id() was not implmented...');
  }

  toBuffer() {
    throw new Error('toBuffer() was not implmented...');
  }

  refs() {
    return [];
  }

  collectRefs(repo) {
    const refsCollection = [];

    function addRefs(object) {
      const refs = object.refs();
      const objs = refs.map(ref => {
        return ref.loadSync(repo);
      });

      refsCollection.push(...refs);
      objs.forEach(obj => addRefs(obj));
    }

    addRefs(this);
    return refsCollection;
  }

  collectRaw(repo) {
    return Promise.all(this.collectRefs(repo).map(ref => ref.loadRaw(repo)));
  }

  asRaw(repo) {
    return repo.loadRaw(this.hash());
  }

  collect(repo) {
    const objects = [];

    function addRefs(object) {
      const objs = object.refs().map(ref => {
        return ref.loadSync(repo);
      });

      objects.push(...objs);
      objs.forEach(obj => addRefs(obj));
    }

    addRefs(this);
    return objects;
  }

  /**
   * indexing method
   */
  hash() {
    return new (_ref || _load_ref()).default(sha1(this.id()));
  }

  compress() {
    return (0, (_utils || _load_utils()).deflate)(this.serialize());
  }

  serialize() {
    const contents = this.toBuffer();
    const header = `${this.constructor.name} ${contents.toString().length}${(_constants || _load_constants()).NULL_BYTE}`;
    return Buffer.concat([new Buffer(header), contents]);
  }

  static parseObject(fileContents, types) {
    return (0, (_utils || _load_utils()).inflate)(fileContents).then(buffer => parse(buffer, types));
  }

  static parseSync(fileContents, types) {
    const buffer = (0, (_zlib || _load_zlib()).inflateSync)(fileContents);
    return parse(buffer, types);
  }
}
exports.default = BitObject;
//# sourceMappingURL=object.js.map