'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path;

function _load_path() {
  return _path = _interopRequireWildcard(require('path'));
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('../repository'));
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('../exceptions');
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

var _invalidBit;

function _load_invalidBit() {
  return _invalidBit = _interopRequireDefault(require('../../consumer/component/exceptions/invalid-bit'));
}

var _s;

function _load_s() {
  return _s = _interopRequireDefault(require('../../consumer/s'));
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../bit-id');
}

var _utils;

function _load_utils() {
  return _utils = require('../../utils');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

class Source extends (_repository || _load_repository()).default {
  getPath() {
    return (_path || _load_path()).join(super.getPath(), (_constants || _load_constants()).BIT_SOURCES_DIRNAME);
  }

  getBitPath(bitName) {
    return (_path || _load_path()).join(this.getPath(), bitName);
  }

  getPartial(name) {
    // @TODO - partial bit does not exist anymore, implement something else to load bits in the sources dir
    // return PartialBit.load(path.join(this.getPath(), name), name, this.scope.name);
  }

  setSource(bit, dependencies) {
    if (!bit.validate()) throw new (_invalidBit || _load_invalidBit()).default();
    return bit.cd(this.composeSourcePath(bit.getId())).write(true).then(() => this.scope.sourcesMap.setBit(bit.getId(), dependencies)).then(() => bit);
  }

  listVersions(bitId) {
    return (0, (_utils || _load_utils()).listDirectories)(this.composeVersionsPath(bitId.name, bitId.box)).map(version => parseInt(version));
  }

  resolveVersion(id) {
    return id.getVersion().resolve(this.listVersions(id));
  }

  loadSource(id) {
    try {
      const version = this.resolveVersion(id);
      return (_s || _load_s()).default.load(this.composeSourcePath({
        name: id.name,
        box: id.box,
        version
      }), id.name, this.scope.name);
    } catch (err) {
      throw new (_exceptions || _load_exceptions()).SourceNotFound(id);
    }
  }

  isBoxEmpty(box) {
    return (0, (_utils || _load_utils()).empty)((0, (_utils || _load_utils()).listDirectories)(this.composeBoxPath(box)));
  }

  composeBoxPath(box) {
    return (_path || _load_path()).join(this.getPath(), box);
  }

  clean(bitId) {
    bitId.version = this.resolveVersion(bitId);
    (0, (_utils || _load_utils()).rmDir)(this.composeSourcePath(bitId));

    if ((0, (_utils || _load_utils()).empty)(this.listVersions(bitId).length)) {
      (0, (_utils || _load_utils()).rmDir)(this.composeVersionsPath(bitId.name, bitId.box));
    }

    if (this.isBoxEmpty(bitId.box)) {
      (0, (_utils || _load_utils()).rmDir)(this.composeBoxPath(bitId.box));
    }
  }

  composeVersionsPath(name, box) {
    return (_path || _load_path()).join(this.getPath(), box, name);
  }

  composeSourcePath(_ref) {
    let name = _ref.name;
    var _ref$box = _ref.box;
    let box = _ref$box === undefined ? 'global' : _ref$box,
        version = _ref.version;

    return (_path || _load_path()).join(this.getPath(), box, name, version.toString());
  }
}
exports.default = Source;
//# sourceMappingURL=source.js.map