'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _uuid;

function _load_uuid() {
  return _uuid = require('uuid');
}

var _path;

function _load_path() {
  return _path = _interopRequireWildcard(require('path'));
}

var _repository;

function _load_repository() {
  return _repository = _interopRequireDefault(require('../repository'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../constants');
}

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Tmp extends (_repository || _load_repository()).default {
  getPath() {
    return (_path || _load_path()).join(super.getPath(), (_constants || _load_constants()).BIT_TMP_DIRNAME);
  }

  composePath(p) {
    return (_path || _load_path()).join(this.getPath(), p);
  }

  save(data) {
    let ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.js';

    return new Promise((resolve, reject) => {
      const fileName = (0, (_uuid || _load_uuid()).v4)();
      const filePath = this.composePath(`${fileName}${ext}`);
      (_fsExtra || _load_fsExtra()).default.outputFile(filePath, data, err => {
        if (err) return reject(err);
        return resolve(filePath);
      });
    });
  }

  saveSync(data) {
    let ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.js';

    const fileName = (0, (_uuid || _load_uuid()).v4)();
    const filePath = this.composePath(`${fileName}${ext}`);
    (_fsExtra || _load_fsExtra()).default.outputFileSync(filePath, data);
    return filePath;
  }

  remove(fileNameOrPath) {
    let ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.js';

    return new Promise((resolve, reject) => {
      const fileName = (_path || _load_path()).parse(fileNameOrPath).name;
      const filePath = this.composePath(`${fileName}${ext}`);
      (_fsExtra || _load_fsExtra()).default.remove(filePath, err => {
        if (err) return reject(err);
        return resolve();
      });
    });
  }

  removeSync(fileNameOrPath) {
    let ext = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '.js';

    const fileName = (_path || _load_path()).parse(fileNameOrPath).name;
    const filePath = this.composePath(`${fileName}${ext}`);
    return (_fsExtra || _load_fsExtra()).default.removeSync(filePath);
  }

  clear() {
    return new Promise((resolve, reject) => {
      (_fsExtra || _load_fsExtra()).default.rmdir(this.getPath(), err => {
        if (err) return reject(err);
        return resolve();
      });
    });
  }

  clearSync() {
    return (_fsExtra || _load_fsExtra()).default.rmdirSync(this.getPath());
  }
}
exports.default = Tmp;
//# sourceMappingURL=tmp.js.map