'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _path;

function _load_path() {
  return _path = _interopRequireWildcard(require('path'));
}

var _fs;

function _load_fs() {
  return _fs = _interopRequireDefault(require('fs'));
}

var _ramda;

function _load_ramda() {
  return _ramda = _interopRequireDefault(require('ramda'));
}

var _ramda2;

function _load_ramda2() {
  return _ramda2 = require('ramda');
}

var _bitJs;

function _load_bitJs() {
  return _bitJs = _interopRequireDefault(require('bit-js'));
}

var _globalConfig;

function _load_globalConfig() {
  return _globalConfig = require('../global-config');
}

var _flattenDependencies;

function _load_flattenDependencies() {
  return _flattenDependencies = require('./flatten-dependencies');
}

var _componentObjects;

function _load_componentObjects() {
  return _componentObjects = _interopRequireDefault(require('./component-objects'));
}

var _component;

function _load_component() {
  return _component = _interopRequireDefault(require('./models/component'));
}

var _remotes;

function _load_remotes() {
  return _remotes = require('../remotes');
}

var _objectRegistrar;

function _load_objectRegistrar() {
  return _objectRegistrar = _interopRequireDefault(require('./object-registrar'));
}

var _utils;

function _load_utils() {
  return _utils = require('../utils');
}

var _constants;

function _load_constants() {
  return _constants = require('../constants');
}

var _scopeJson;

function _load_scopeJson() {
  return _scopeJson = require('./scope-json');
}

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _repositories;

function _load_repositories() {
  return _repositories = require('./repositories');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../bit-id');
}

var _component2;

function _load_component2() {
  return _component2 = _interopRequireDefault(require('../consumer/component'));
}

var _componentVersion;

function _load_componentVersion() {
  return _componentVersion = _interopRequireDefault(require('./component-version'));
}

var _objects;

function _load_objects() {
  return _objects = require('./objects');
}

var _componentDependencies;

function _load_componentDependencies() {
  return _componentDependencies = _interopRequireDefault(require('./component-dependencies'));
}

var _versionDependencies;

function _load_versionDependencies() {
  return _versionDependencies = _interopRequireDefault(require('./version-dependencies'));
}

var _sources;

function _load_sources() {
  return _sources = _interopRequireDefault(require('./repositories/sources'));
}

var _hooks;

function _load_hooks() {
  return _hooks = require('../hooks');
}

var _npm;

function _load_npm() {
  return _npm = _interopRequireDefault(require('../utils/npm'));
}

var _consumer;

function _load_consumer() {
  return _consumer = _interopRequireDefault(require('../consumer/consumer'));
}

var _indexer;

function _load_indexer() {
  return _indexer = require('../search/indexer');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

const removeNils = (_ramda || _load_ramda()).default.reject((_ramda || _load_ramda()).default.isNil);
const pathHasScope = (0, (_utils || _load_utils()).pathHas)([(_constants || _load_constants()).OBJECTS_DIR, (_constants || _load_constants()).BIT_HIDDEN_DIR]);

class Scope {

  constructor(scopeProps) {
    this.created = false;

    this.path = scopeProps.path;
    this.scopeJson = scopeProps.scopeJson;
    this.created = scopeProps.created || false;
    this.tmp = scopeProps.tmp || new (_repositories || _load_repositories()).Tmp(this);
    this.sources = scopeProps.sources || new (_sources || _load_sources()).default(this);
    this.objects = scopeProps.objects || new (_objects || _load_objects()).Repository(this, (0, (_objectRegistrar || _load_objectRegistrar()).default)());
  }

  get groupName() {
    if (!this.scopeJson.groupName) return null;
    return this.scopeJson.groupName;
  }

  get name() {
    return this.scopeJson.name;
  }

  getPath() {
    return this.path;
  }

  remotes() {
    const self = this;
    function mergeRemotes(globalRemotes) {
      const globalObj = globalRemotes.toPlainObject();
      return (_remotes || _load_remotes()).Remotes.load((0, (_ramda2 || _load_ramda2()).merge)(globalObj, self.scopeJson.remotes));
    }

    return (_globalConfig || _load_globalConfig()).GlobalRemotes.load().then(mergeRemotes);
  }

  describe() {
    return {
      name: this.name
    };
  }

  toConsumerComponents(components) {
    return Promise.all(components.map(c => c.toConsumerComponent(c.latest().toString(), this.name, this.objects)));
  }

  list() {
    // @Deprecated
    return this.objects.list() // @TODO - check version and cross check them with components
    .then(components => this.toConsumerComponents(components));
  }

  listStage() {
    return this.objects.list().then(components => this.toConsumerComponents(components.filter(c => c.scope === this.name)));
  }

  getDependencies() {}

  put(consumerComponent, message, force, loader) {
    consumerComponent.scope = this.name;
    if (loader) {
      loader.text = 'importing components';
      loader.start();
    }

    return this.importMany(consumerComponent.dependencies).then(dependencies => {
      return (0, (_flattenDependencies || _load_flattenDependencies()).flattenDependencyIds)(dependencies, this.objects).then(depIds => {
        return this.sources.addSource({
          source: consumerComponent, depIds, message, force, loader
        }).then(component => {
          if (loader) {
            loader.text = 'persisting data';
          }
          return this.objects.persist().then(() => component.toVersionDependencies((_constants || _load_constants()).LATEST, this, this.name)).then(deps => deps.toConsumer(this.objects));
        });
      });
    });
  }

  importSrc(componentObjects) {
    return this.sources.merge(componentObjects.toObjects(this.objects)).then(() => this.objects.persist());
  }

  export(componentObjects) {
    const objects = componentObjects.toObjects(this.objects);
    const component = objects.component;

    return this.sources.merge(objects, true).then(() => this.objects.persist()).then(() => component.toComponentVersion((_constants || _load_constants()).LATEST, this.name)).then(compVersion => this.getObjects([compVersion.id]).then(objs => {
      return compVersion.toConsumer(this.objects).then(consumerComponent => (0, (_indexer || _load_indexer()).index)(consumerComponent, this.getPath())).then(consumerComponent => (0, (_hooks || _load_hooks()).postExportHook)({ id: consumerComponent.id.toString() })).then(() => (0, (_utils || _load_utils()).first)(objs));
    }));
  }

  getExternalOnes(ids, remotes) {
    let localFetch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    return this.sources.getMany(ids).then(defs => {
      if (localFetch) {
        return Promise.all(defs.map(def => {
          return def.component.toComponentVersion(def.id.version, this.name);
        }));
      }

      return remotes.fetch(defs.map(def => def.id), this, true).then(componentObjects => {
        return Promise.all(componentObjects.map(compObj => this.importSrc(compObj)));
      }).then(() => this.getExternalOnes(ids, remotes, true));
    });
  }

  getExternalMany(ids, remotes) {
    let localFetch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    return this.sources.getMany(ids).then(defs => {
      const left = defs.filter(def => !def.component && localFetch);
      if (left.length === 0) {
        return Promise.all(defs.map(def => def.component.toVersionDependencies(def.id.version, this, def.id.scope)));
      }

      return remotes.fetch(left.map(def => def.id), this).then(componentObjects => {
        return Promise.all(componentObjects.map(compObj => this.importSrc(compObj)));
      }).then(() => this.getExternalMany(ids, remotes));
    });
  }

  getExternal(id, remotes) {
    let localFetch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    return this.sources.get(id).then(component => {
      if (component && localFetch) {
        return component.toVersionDependencies(id.version, this, id.scope);
      }
      return remotes.fetch([id], this).then((_ref) => {
        var _ref2 = _slicedToArray(_ref, 1);

        let componentObjects = _ref2[0];

        return this.importSrc(componentObjects);
      }).then(() => this.getExternal(id, remotes));
    });
  }

  getExternalOne(id, remotes) {
    let localFetch = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;

    return this.sources.get(id).then(component => {
      if (component && localFetch) return component.toComponentVersion(id.version, this.name);
      return remotes.fetch([id], this, true).then((_ref3) => {
        var _ref4 = _slicedToArray(_ref3, 1);

        let componentObjects = _ref4[0];
        return this.importSrc(componentObjects);
      }).then(() => this.getExternal(id, remotes));
    });
  }

  getObjects(ids) {
    return this.importMany(ids).then(versions => Promise.all(versions.map(version => version.toObjects(this.objects))));
  }

  getObject(hash) {
    return new (_objects || _load_objects()).Ref(hash).load(this.objects);
  }

  importMany(ids) {
    const idsWithoutNils = removeNils(ids);
    if ((_ramda || _load_ramda()).default.isEmpty(idsWithoutNils)) return Promise.resolve([]);

    var _splitWhen = (0, (_ramda2 || _load_ramda2()).splitWhen)(id => id.isLocal(this.name), idsWithoutNils),
        _splitWhen2 = _slicedToArray(_splitWhen, 2);

    const externals = _splitWhen2[0],
          locals = _splitWhen2[1];


    return this.sources.getMany(locals).then(localDefs => {
      return Promise.all(localDefs.map(def => {
        if (!def.component) throw new (_exceptions || _load_exceptions()).ComponentNotFound(def.id);
        return def.component.toVersionDependencies(def.id.version, this, def.id.scope);
      })).then(versionDeps => {
        return this.remotes().then(remotes => this.getExternalMany(externals, remotes)).then(externalDeps => versionDeps.concat(externalDeps));
      });
    });
  }

  import(id) {
    if (!id.isLocal(this.name)) {
      return this.remotes().then(remotes => this.getExternal(id, remotes));
    }

    return this.sources.get(id).then(component => {
      if (!component) throw new (_exceptions || _load_exceptions()).ComponentNotFound(id);
      return (0, (_hooks || _load_hooks()).postImportHook)({ id: id.toString() }).then(() => component.toVersionDependencies(id.version, this));
    });
  }

  get(id) {
    return this.import(id).then(versionDependencies => {
      return versionDependencies.toConsumer(this.objects);
    });
  }

  getMany(ids) {
    const idsWithoutNils = removeNils(ids);
    if ((_ramda || _load_ramda()).default.isEmpty(idsWithoutNils)) return Promise.resolve([]);
    return this.importMany(idsWithoutNils).then(versionDependenciesArr => {
      return Promise.all(versionDependenciesArr.map(versionDependencies => versionDependencies.toConsumer(this.objects)));
    });
  }

  // @TODO optimize ASAP
  modify(id) {
    return this.import(id, true).then(versionDependencies => {
      const versions = versionDependencies.component.component.listVersions();
      const versionsP = this.importManyOnes(versions.map(version => {
        const versionId = (_bitId || _load_bitId()).BitId.parse(id.toString());
        versionId.version = version.toString();
        return versionId;
      }));

      return Promise.all([versionDependencies.toObjects(this.objects), versionsP]);
    }).then((_ref5) => {
      var _ref6 = _slicedToArray(_ref5, 1);

      let componentObjects = _ref6[0];

      return this.export(componentObjects);
    }).then(() => {
      id.scope = this.name;
      return this.get(id);
    });
  }

  loadComponent(id) {
    if (!id.isLocal(this.name)) {
      throw new Error('cannot load bit from remote scope, please import first');
    }

    return this.getOne(id).then(component => {
      if (!component) throw new (_exceptions || _load_exceptions()).ComponentNotFound(id);
      return component.toConsumer(this.objects);
    });
  }

  loadComponentLogs(id) {
    return this.sources.get(id).then(componentModel => {
      if (!componentModel) throw new (_exceptions || _load_exceptions()).ComponentNotFound(id);
      return componentModel.collectLogs(this.objects);
    });
  }

  loadAllVersions(id) {
    return this.sources.get(id).then(componentModel => {
      if (!componentModel) throw new (_exceptions || _load_exceptions()).ComponentNotFound(id);
      return componentModel.collectVersions(this.objects);
    });
  }

  getOne(id) {
    if (!id.isLocal(this.name)) {
      return this.remotes().then(remotes => this.getExternalOne(id, remotes));
    }

    return this.sources.get(id).then(component => {
      if (!component) throw new (_exceptions || _load_exceptions()).ComponentNotFound(id);
      return component.toComponentVersion(id.version, this.name);
    });
  }

  importManyOnes(ids) {
    var _splitWhen3 = (0, (_ramda2 || _load_ramda2()).splitWhen)(id => id.isLocal(this.name), ids),
        _splitWhen4 = _slicedToArray(_splitWhen3, 2);

    const externals = _splitWhen4[0],
          locals = _splitWhen4[1];


    return this.sources.getMany(locals).then(localDefs => {
      return Promise.all(localDefs.map(def => {
        if (!def.component) throw new (_exceptions || _load_exceptions()).ComponentNotFound(def.id);
        return def.component.toComponentVersion(def.id.version, this.name);
      })).then(versionDeps => {
        return this.remotes().then(remotes => this.getExternalOnes(externals, remotes)).then(externalDeps => versionDeps.concat(externalDeps));
      });
    });
  }

  manyOneObjects(ids) {
    return this.importManyOnes(ids).then(componentVersions => Promise.all(componentVersions.map(version => {
      return version.toObjects(this.objects);
    })));
  }

  exportAction(bitId, remoteName) {
    return this.remotes().then(remotes => {
      return remotes.resolve(remoteName, this).then(remote => {
        return this.sources.getObjects(bitId).then(component => remote.push(component).then(objects => this.clean(bitId).then(() => objects)).then(objects => this.importSrc(objects)).then(() => {
          bitId.scope = remoteName;
          return this.get(bitId);
        }));
      });
    });
  }

  ensureDir() {
    return this.tmp.ensureDir().then(() => this.scopeJson.write(this.getPath())).then(() => this.objects.ensureDir()).then(() => this);
  }

  clean(bitId) {
    return this.sources.clean(bitId);
  }

  /**
   * sync method that loads the environment/(path to environment component)
   */
  loadEnvironment(bitId, opts) {
    if (opts && opts.pathOnly) {
      try {
        return (_bitJs || _load_bitJs()).default.loadExact(bitId.toString(), this.getPath(), opts);
      } catch (e) {
        throw new (_exceptions || _load_exceptions()).ResolutionException(e.message);
      }
    }

    try {
      return (_bitJs || _load_bitJs()).default.loadExact(bitId.toString(), this.getPath());
    } catch (e) {
      throw new (_exceptions || _load_exceptions()).ResolutionException(e.message);
    }
  }

  installEnvironment(_ref7) {
    let ids = _ref7.ids,
        consumer = _ref7.consumer,
        verbose = _ref7.verbose,
        loader = _ref7.loader;

    const installPackageDependencies = component => {
      const scopePath = this.getPath();
      const nodeModulesDir = consumer ? (_path || _load_path()).dirname(scopePath) : scopePath;
      const deps = component.packageDependencies;

      if (verbose && loader) loader.stop(); // in order to show npm install output on verbose flag
      if (loader) loader.text = 'ensuring npm dependencies';

      return Promise.all((_ramda || _load_ramda()).default.values((_ramda || _load_ramda()).default.mapObjIndexed((value, key) => (0, (_npm || _load_npm()).default)({ name: key, version: value, dir: nodeModulesDir, silent: !verbose }), deps))).then(() => component);
    };

    return this.getMany(ids).then(components => {
      if (consumer) {
        return consumer.writeToComponentsDir((0, (_utils || _load_utils()).flatten)(components));
      } else {
        throw new Error('can install env component in bare scope');
        // TODO -- write in scope relevant directory !! 
      }
    }).then(components => {
      return Promise.all(components.map(installPackageDependencies));
    });
  }

  runComponentSpecs(id) {
    if (!id.isLocal(this.name)) {
      throw new Error('cannot run specs on remote scopes');
    }

    return this.loadComponent(id).then(component => {
      return component.runSpecs(this);
    });
  }

  build(bitId) {
    return this.loadComponent(bitId).then(component => component.build(this));
  }

  static ensure() {
    let path = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : process.cwd();
    let name = arguments[1];
    let groupName = arguments[2];

    if (pathHasScope(path)) return this.load(path);
    if (!name) name = (0, (_utils || _load_utils()).currentDirName)();
    const scopeJson = new (_scopeJson || _load_scopeJson()).ScopeJson({ name, groupName });
    return Promise.resolve(new Scope({ path, created: true, scopeJson }));
  }

  static load(absPath) {
    let scopePath = (0, (_utils || _load_utils()).propogateUntil)(absPath, pathHasScope);
    if (!scopePath) throw new (_exceptions || _load_exceptions()).ScopeNotFound();
    if ((_fs || _load_fs()).default.existsSync((_path || _load_path()).join(scopePath, (_constants || _load_constants()).BIT_HIDDEN_DIR))) {
      scopePath = (_path || _load_path()).join(scopePath, (_constants || _load_constants()).BIT_HIDDEN_DIR);
    }
    const path = scopePath;

    return (0, (_utils || _load_utils()).readFile)((0, (_scopeJson || _load_scopeJson()).getPath)(scopePath)).then(rawScopeJson => {
      const scopeJson = (_scopeJson || _load_scopeJson()).ScopeJson.loadFromJson(rawScopeJson.toString());
      const scope = new Scope({ path, scopeJson });
      return scope;
    });
  }
}
exports.default = Scope;
//# sourceMappingURL=scope.js.map