'use strict';

var _serverlessIndex;

function _load_serverlessIndex() {
  return _serverlessIndex = _interopRequireDefault(require('./serverless-index'));
}

var _indexer;

function _load_indexer() {
  return _indexer = _interopRequireDefault(require('./indexer'));
}

var _searcher;

function _load_searcher() {
  return _searcher = _interopRequireDefault(require('./searcher'));
}

var _consumer;

function _load_consumer() {
  return _consumer = require('../consumer');
}

var _scope;

function _load_scope() {
  return _scope = require('../scope');
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function searchLocally(queryStr) {
  let reindex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

  return new Promise((resolve, reject) => {
    let scopePath;
    if (reindex) {
      (0, (_consumer || _load_consumer()).loadConsumer)().then(consumer => {
        scopePath = consumer.scope.path;
        return consumer.scope.listStage();
      }).then(components => {
        return (_indexer || _load_indexer()).default.indexAll(scopePath, components);
      }).then(() => {
        resolve((_searcher || _load_searcher()).default.search(queryStr, scopePath));
      });
    } else {
      (0, (_consumer || _load_consumer()).loadConsumer)().then(consumer => {
        scopePath = consumer.scope.path;
        resolve((_searcher || _load_searcher()).default.search(queryStr, scopePath));
      });
    }
  });
}


function searchRemotely(queryStr, scope) {
  let reindex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

  return new Promise((resolve, reject) => {
    (0, (_consumer || _load_consumer()).loadConsumer)().then(consumer => {
      return consumer.scope.remotes().then(remotes =>
      // $FlowFixMe
      remotes.resolve(scope, consumer.scope.name).then(remote => {
        resolve(remote.search(queryStr, reindex));
      }));
    });
  });
}

function scopeSearch(path, query, reindex) {
  return new Promise((resolve, reject) => {
    if (reindex) {
      (0, (_scope || _load_scope()).loadScope)(path).then(scope => {
        return scope.listStage();
      }).then(components => {
        return (_indexer || _load_indexer()).default.indexAll(path, components);
      }).then(() => {
        resolve((_searcher || _load_searcher()).default.search(query, path));
      });
    } else {
      resolve((_searcher || _load_searcher()).default.search(query, path));
    }
  });
}

module.exports = {
  searchLocally,
  searchRemotely,
  scopeSearch
};
//# sourceMappingURL=search-adapter.js.map