'use strict';

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _fsExtra;

function _load_fsExtra() {
  return _fsExtra = _interopRequireDefault(require('fs-extra'));
}

var _searchIndex;

function _load_searchIndex() {
  return _searchIndex = _interopRequireDefault(require('search-index'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const indexName = 'search_index';

const logLevel = 'error';
let index;

function getIndexPath(scopePath) {
  return (_path || _load_path()).default.join(scopePath, indexName);
}

function deleteDb(scopePath) {
  const indexPath = getIndexPath(scopePath);
  if (!scopePath || !indexPath) return;
  (_fsExtra || _load_fsExtra()).default.removeSync(indexPath);
}

function initializeIndex(scopePath) {
  if (!index) {
    // static var to make sure the index is not instantiated twice
    const indexOptions = {
      indexPath: getIndexPath(scopePath),
      logLevel,
      stopwords: []
    };

    index = new Promise((resolve, reject) => {
      (0, (_searchIndex || _load_searchIndex()).default)(indexOptions, (err, si) => {
        if (err) reject(err);
        resolve(si);
      });
    });
  }

  return index;
}

module.exports = {
  deleteDb,
  getIndexPath,
  initializeIndex
};
//# sourceMappingURL=serverless-index.js.map