'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _exceptions;

function _load_exceptions() {
  return _exceptions = require('./exceptions');
}

var _versionParser;

function _load_versionParser() {
  return _versionParser = _interopRequireDefault(require('./version-parser'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Version {

  constructor(versionNum, latest) {
    this.versionNum = versionNum;
    this.latest = latest;
  }

  increase() {
    if (!this.versionNum) throw new (_exceptions || _load_exceptions()).InvalidVersionChange();
    this.versionNum = this.versionNum + 1;
    return this;
  }

  decrease() {
    if (!this.versionNum || this.versionNum <= 1) throw new (_exceptions || _load_exceptions()).InvalidVersionChange();
    this.versionNum = this.versionNum - 1;
    return this;
  }

  resolve(availableVersion) {
    const getLatest = () => Math.max(...availableVersion);

    if (this.latest) return getLatest();
    return this.versionNum;
  }

  toString() {
    if (!this.versionNum && this.latest) return 'latest';
    if (this.versionNum && this.latest) return `*${this.versionNum}`;
    if (this.versionNum && !this.latest) return this.versionNum.toString();
    throw new (_exceptions || _load_exceptions()).InvalidVersion();
  }

  static parse(versionStr) {
    return (0, (_versionParser || _load_versionParser()).default)(versionStr);
  }

  static validate(versionStr) {
    try {
      (0, (_versionParser || _load_versionParser()).default)(versionStr);
      return true;
    } catch (err) {
      return false;
    }
  }
}
exports.default = Version;
//# sourceMappingURL=version.js.map