'use strict';

var _chai;

function _load_chai() {
  return _chai = require('chai');
}

var _version;

function _load_version() {
  return _version = _interopRequireDefault(require('./version'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('Version', () => {
  describe('toString()', () => {
    it('should return a latest tested version', () => {
      const version = new (_version || _load_version()).default(10, true);
      (0, (_chai || _load_chai()).expect)(version.toString()).to.equal('*10');
    });

    it('should return latest', () => {
      const version = new (_version || _load_version()).default(null, true);
      (0, (_chai || _load_chai()).expect)(version.toString()).to.equal('latest');
    });

    it('should return concerete version number', () => {
      const version = new (_version || _load_version()).default(12, false);
      (0, (_chai || _load_chai()).expect)(version.toString()).to.equal('12');
    });

    it('should throw an invalid version exception', () => {
      const version = new (_version || _load_version()).default(null, false);
      (0, (_chai || _load_chai()).expect)(() => {
        version.toString();
      }).to.throw();
    });
  });

  describe('increase() + decrease()', () => {
    it('should increase concerete version by one', () => {
      const version = new (_version || _load_version()).default(24, false);
      version.increase();
      (0, (_chai || _load_chai()).expect)(version.versionNum).to.equal(25);
    });

    it('should increase latest tested version by one', () => {
      const version = new (_version || _load_version()).default(24, true);
      version.increase();
      (0, (_chai || _load_chai()).expect)(version.versionNum).to.equal(25);
    });

    it('should decrease concerete version by one', () => {
      const version = new (_version || _load_version()).default(24, false);
      version.decrease();
      (0, (_chai || _load_chai()).expect)(version.versionNum).to.equal(23);
    });

    it('should decrease latest tested version by one', () => {
      const version = new (_version || _load_version()).default(24, true);
      version.decrease();
      (0, (_chai || _load_chai()).expect)(version.versionNum).to.equal(23);
    });

    it('should throw an InvalidVersionChange error when trying to increase or decrease latest', () => {
      const version = new (_version || _load_version()).default(null, true);
      (0, (_chai || _load_chai()).expect)(() => {
        version.increase();
      }).to.throw();

      (0, (_chai || _load_chai()).expect)(() => {
        version.decrease();
      }).to.throw();
    });
  });

  describe('parse()', () => {
    it('should parse given version as latest', () => {
      const version = (_version || _load_version()).default.parse('latest');
      (0, (_chai || _load_chai()).expect)(version.versionNum).to.equal(null);
      (0, (_chai || _load_chai()).expect)(version.latest).to.equal(true);
    });
  });
});
//# sourceMappingURL=version.spec.js.map