/** @flow */
import R from 'ramda';
import Command from '../../command';
import { importAction } from '../../../api/consumer';
import { immutableUnshift } from '../../../utils';
import { formatBit, paintHeader } from '../../chalk-box';

export default class Import extends Command {
  name = 'import [ids]';
  description = 'import a component';
  alias = 'i';
  opts = [
    ['s', 'save', 'save into bit.json'],
    ['t', 'tester', 'import a tester environment component'],
    ['v', 'verbose', 'show a more verbose output when possible'],
    ['c', 'compiler', 'import a compiler environment component'],
    ['p', 'prefix', 'import components into a specific directory'],
    ['d', 'dev', 'also import dev dependencies (compiler | tester)']
  ];
  loader = { autoStart: false, text: 'importing components' };

  action([id, ]: [string, ], { save, tester, compiler, verbose, prefix, dev }: any): Promise<any> {
    if (prefix) { return Promise.reject(new Error('prefix option currently not supported')); }
    // TODO - prefix returns true instead of the relevant string.
    const loader = this.loader;
    // @TODO - import should support multiple components
    if (tester && compiler) {
      throw new Error('you cant use tester and compiler flags combined');
    }
    
    return importAction({ bitId: id, save, tester, compiler, loader, verbose, prefix, dev });
  }

  report(components: any): string {
    if (R.isEmpty(components)) { return 'done'; }
    return immutableUnshift(
      components.map(formatBit),
      paintHeader('imported the following components:')
    ).join('\n');
  }
}
